"use strict";
(() => {
  var __typeError = (msg) => {
    throw TypeError(msg);
  };
  var __accessCheck = (obj, member, msg) => member.has(obj) || __typeError("Cannot " + msg);
  var __privateGet = (obj, member, getter) => (__accessCheck(obj, member, "read from private field"), getter ? getter.call(obj) : member.get(obj));
  var __privateAdd = (obj, member, value) => member.has(obj) ? __typeError("Cannot add the same private member more than once") : member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
  var __privateSet = (obj, member, value, setter) => (__accessCheck(obj, member, "write to private field"), setter ? setter.call(obj, value) : member.set(obj, value), value);
  var __privateMethod = (obj, member, method) => (__accessCheck(obj, member, "access private method"), method);
  var __privateWrapper = (obj, member, setter, getter) => ({
    set _(value) {
      __privateSet(obj, member, value, setter);
    },
    get _() {
      return __privateGet(obj, member, getter);
    }
  });

  // pkg/lib/cockpit/_internal/base64.js
  function uint6_to_b64(x) {
    return x < 26 ? x + 65 : x < 52 ? x + 71 : x < 62 ? x - 4 : x === 62 ? 43 : x === 63 ? 47 : 65;
  }
  function base64_encode(data) {
    if (typeof data === "string")
      return window.btoa(data);
    if (data instanceof window.ArrayBuffer)
      data = new window.Uint8Array(data);
    const length = data.length;
    let mod3 = 2;
    let str = "";
    for (let uint24 = 0, i = 0; i < length; i++) {
      mod3 = i % 3;
      uint24 |= data[i] << (16 >>> mod3 & 24);
      if (mod3 === 2 || length - i === 1) {
        str += String.fromCharCode(
          uint6_to_b64(uint24 >>> 18 & 63),
          uint6_to_b64(uint24 >>> 12 & 63),
          uint6_to_b64(uint24 >>> 6 & 63),
          uint6_to_b64(uint24 & 63)
        );
        uint24 = 0;
      }
    }
    return str.substring(0, str.length - 2 + mod3) + (mod3 === 2 ? "" : mod3 === 1 ? "=" : "==");
  }
  function b64_to_uint6(x) {
    return x > 64 && x < 91 ? x - 65 : x > 96 && x < 123 ? x - 71 : x > 47 && x < 58 ? x + 4 : x === 43 ? 62 : x === 47 ? 63 : 0;
  }
  function base64_decode(str, constructor) {
    if (constructor === String)
      return window.atob(str);
    const ilen = str.length;
    let eq;
    for (eq = 0; eq < 3; eq++) {
      if (str[ilen - (eq + 1)] != "=")
        break;
    }
    const olen = (ilen * 3 + 1 >> 2) - eq;
    const data = new (constructor || Array)(olen);
    for (let mod3, mod4, uint24 = 0, oi = 0, ii = 0; ii < ilen; ii++) {
      mod4 = ii & 3;
      uint24 |= b64_to_uint6(str.charCodeAt(ii)) << 18 - 6 * mod4;
      if (mod4 === 3 || ilen - ii === 1) {
        for (mod3 = 0; mod3 < 3 && oi < olen; mod3++, oi++)
          data[oi] = uint24 >>> (16 >>> mod3 & 24) & 255;
        uint24 = 0;
      }
    }
    return data;
  }

  // pkg/lib/cockpit/_internal/common.ts
  if (typeof window.debugging === "undefined") {
    try {
      Object.defineProperty(window, "debugging", {
        get: function() {
          return window.sessionStorage.debugging || window.localStorage.debugging;
        },
        set: function(x) {
          window.sessionStorage.debugging = x;
        }
      });
    } catch {
    }
  }
  function in_array(array, val) {
    const length = array.length;
    for (let i = 0; i < length; i++) {
      if (val === array[i])
        return true;
    }
    return false;
  }
  function is_function(x) {
    return typeof x === "function";
  }
  function is_object(x) {
    return x !== null && typeof x === "object";
  }
  function is_plain_object(x) {
    return is_object(x) && Object.prototype.toString.call(x) === "[object Object]";
  }
  function invoke_functions(functions, self, args) {
    const length = functions?.length ?? 0;
    for (let i = 0; i < length; i++) {
      if (functions[i])
        functions[i].apply(self, args);
    }
  }
  function iterate_data(data, callback, batch = 64 * 1024) {
    if (typeof data === "string") {
      for (let i = 0; i < data.length; i += batch) {
        callback(data.substring(i, i + batch));
      }
    } else if (data) {
      for (let i = 0; i < data.byteLength; i += batch) {
        const n = Math.min(data.byteLength - i, batch);
        callback(new Uint8Array(data.buffer, i, n));
      }
    }
  }
  function join_data(buffers, binary) {
    if (!binary)
      return buffers.join("");
    let total = 0;
    const length = buffers.length;
    for (let i = 0; i < length; i++)
      total += buffers[i].length;
    const data = new Uint8Array(total);
    for (let j = 0, i = 0; i < length; i++) {
      data.set(buffers[i], j);
      j += buffers[i].length;
    }
    return data;
  }

  // pkg/lib/cockpit/_internal/deferred.js
  var later_queue = [];
  var later_timeout = null;
  function later_drain() {
    const queue = later_queue;
    later_timeout = null;
    later_queue = [];
    for (; ; ) {
      const func = queue.shift();
      if (!func)
        break;
      func();
    }
  }
  function later_invoke(func) {
    if (func)
      later_queue.push(func);
    if (later_timeout === null)
      later_timeout = window.setTimeout(later_drain, 0);
  }
  function promise_then(state, fulfilled, rejected, updated) {
    if (fulfilled === void 0 && rejected === void 0 && updated === void 0)
      return null;
    const result = new Deferred();
    state.pending = state.pending || [];
    state.pending.push([result, fulfilled, rejected, updated]);
    if (state.status > 0)
      schedule_process_queue(state);
    return result.promise;
  }
  function create_promise(state) {
    const self = function Promise2(target) {
      if (target) {
        Object.assign(target, self);
        return target;
      }
      return self;
    };
    state.status = 0;
    self.then = function then(fulfilled, rejected, updated) {
      return promise_then(state, fulfilled, rejected, updated) || self;
    };
    self.catch = function catch_(callback) {
      return promise_then(state, null, callback) || self;
    };
    self.finally = function finally_(callback, updated) {
      return promise_then(state, function() {
        return handle_callback(arguments, true, callback);
      }, function() {
        return handle_callback(arguments, false, callback);
      }, updated) || self;
    };
    self.done = function done(fulfilled) {
      promise_then(state, fulfilled);
      return self;
    };
    self.fail = function fail(rejected) {
      promise_then(state, null, rejected);
      return self;
    };
    self.always = function always(callback) {
      promise_then(state, callback, callback);
      return self;
    };
    self.progress = function progress(updated) {
      promise_then(state, null, null, updated);
      return self;
    };
    self.state = function state_() {
      if (state.status == 1)
        return "resolved";
      if (state.status == 2)
        return "rejected";
      return "pending";
    };
    self.promise = self;
    return self;
  }
  function process_queue(state) {
    const pending = state.pending;
    state.process_scheduled = false;
    state.pending = void 0;
    for (let i = 0, ii = pending.length; i < ii; ++i) {
      state.pur = true;
      const deferred = pending[i][0];
      const fn = pending[i][state.status];
      if (is_function(fn)) {
        deferred.resolve(fn.apply(state.promise, state.values));
      } else if (state.status === 1) {
        deferred.resolve.apply(deferred.resolve, state.values);
      } else {
        deferred.reject.apply(deferred.reject, state.values);
      }
    }
  }
  function schedule_process_queue(state) {
    if (state.process_scheduled || !state.pending)
      return;
    state.process_scheduled = true;
    later_invoke(function() {
      process_queue(state);
    });
  }
  function deferred_resolve(state, values) {
    let then;
    let done = false;
    if (is_object(values[0]) || is_function(values[0]))
      then = values[0]?.then;
    if (is_function(then)) {
      state.status = -1;
      then.call(values[0], function() {
        if (done)
          return;
        done = true;
        deferred_resolve(state, arguments);
      }, function() {
        if (done)
          return;
        done = true;
        deferred_reject(state, arguments);
      }, function() {
        deferred_notify(state, arguments);
      });
    } else {
      state.values = values;
      state.status = 1;
      schedule_process_queue(state);
    }
  }
  function deferred_reject(state, values) {
    state.values = values;
    state.status = 2;
    schedule_process_queue(state);
  }
  function deferred_notify(state, values) {
    const callbacks = state.pending;
    if (state.status <= 0 && callbacks?.length) {
      later_invoke(function() {
        for (let i = 0, ii = callbacks.length; i < ii; i++) {
          const result = callbacks[i][0];
          const callback = callbacks[i][3];
          if (is_function(callback))
            result.notify(callback.apply(state.promise, values));
          else
            result.notify.apply(result, values);
        }
      });
    }
  }
  function Deferred() {
    const self = this;
    const state = {};
    self.promise = state.promise = create_promise(state);
    self.resolve = function resolve() {
      if (arguments[0] === state.promise)
        throw new Error("Expected promise to be resolved with other value than itself");
      if (!state.status)
        deferred_resolve(state, arguments);
      return self;
    };
    self.reject = function reject() {
      if (state.status)
        return;
      deferred_reject(state, arguments);
      return self;
    };
    self.notify = function notify() {
      deferred_notify(state, arguments);
      return self;
    };
  }
  function prep_promise(values, resolved) {
    const result = new Deferred();
    if (resolved)
      result.resolve.apply(result, values);
    else
      result.reject.apply(result, values);
    return result.promise;
  }
  function handle_callback(values, is_resolved, callback) {
    let callback_output = null;
    if (is_function(callback))
      callback_output = callback();
    if (callback_output && is_function(callback_output.then)) {
      return callback_output.then(function() {
        return prep_promise(values, is_resolved);
      }, function() {
        return prep_promise(arguments, false);
      });
    } else {
      return prep_promise(values, is_resolved);
    }
  }

  // pkg/lib/cockpit/_internal/event-mixin.js
  function event_mixin(obj, handlers) {
    Object.defineProperties(obj, {
      addEventListener: {
        enumerable: false,
        value: function addEventListener(type, handler) {
          if (handlers[type] === void 0)
            handlers[type] = [];
          handlers[type].push(handler);
        }
      },
      removeEventListener: {
        enumerable: false,
        value: function removeEventListener(type, handler) {
          const length = handlers[type] ? handlers[type].length : 0;
          for (let i = 0; i < length; i++) {
            if (handlers[type][i] === handler) {
              handlers[type][i] = null;
              break;
            }
          }
        }
      },
      dispatchEvent: {
        enumerable: false,
        value: function dispatchEvent(event) {
          let type, args;
          if (typeof event === "string") {
            type = event;
            args = Array.prototype.slice.call(arguments, 1);
            let detail = null;
            if (arguments.length == 2)
              detail = arguments[1];
            else if (arguments.length > 2)
              detail = args;
            event = new CustomEvent(type, {
              bubbles: false,
              cancelable: false,
              detail
            });
            args.unshift(event);
          } else {
            type = event.type;
            args = arguments;
          }
          if (is_function(obj["on" + type]))
            obj["on" + type].apply(obj, args);
          invoke_functions(handlers[type], obj, args);
        }
      }
    });
  }

  // pkg/lib/cockpit/event.ts
  var _listeners;
  var EventEmitter = class {
    constructor() {
      __privateAdd(this, _listeners, {});
    }
    on(event, listener) {
      const listeners = __privateGet(this, _listeners)[event] ||= [];
      listeners.push(listener);
      return () => {
        listeners.splice(listeners.indexOf(listener), 1);
      };
    }
    emit(event, ...args) {
      for (const listener of __privateGet(this, _listeners)[event] || []) {
        listener(...args);
      }
    }
  };
  _listeners = new WeakMap();

  // pkg/lib/cockpit/_internal/location-utils.ts
  function get_url_root() {
    const meta_url_root = document.head.querySelector("meta[name='url-root']");
    if (meta_url_root instanceof HTMLMetaElement) {
      return meta_url_root.content.replace(/^\/+|\/+$/g, "");
    } else {
      try {
        return window.localStorage.getItem("url-root");
      } catch {
        return null;
      }
    }
  }
  var url_root = get_url_root();
  var transport_origin = window.location.origin;
  function calculate_application() {
    let path = window.location.pathname || "/";
    let _url_root = url_root;
    if (window.mock?.pathname)
      path = window.mock.pathname;
    if (window.mock?.url_root)
      _url_root = window.mock.url_root;
    if (_url_root && path.indexOf("/" + _url_root) === 0)
      path = path.replace("/" + _url_root, "") || "/";
    if (path.indexOf("/cockpit/") !== 0 && path.indexOf("/cockpit+") !== 0) {
      if (path.indexOf("/=") === 0)
        path = "/cockpit+" + path.split("/")[1];
      else
        path = "/cockpit";
    }
    return path.split("/")[1];
  }
  function calculate_url(suffix) {
    if (!suffix)
      suffix = "socket";
    const window_loc = window.location.toString();
    let _url_root = url_root;
    if (window.mock?.url)
      return window.mock.url;
    if (window.mock?.url_root)
      _url_root = window.mock.url_root;
    let prefix = calculate_application();
    if (_url_root)
      prefix = _url_root + "/" + prefix;
    if (window_loc.indexOf("http:") === 0) {
      return "ws://" + window.location.host + "/" + prefix + "/" + suffix;
    } else if (window_loc.indexOf("https:") === 0) {
      return "wss://" + window.location.host + "/" + prefix + "/" + suffix;
    } else {
      throw new Error("Cockpit must be used over http or https");
    }
  }

  // pkg/lib/cockpit/_internal/parentwebsocket.ts
  var ParentWebSocket = class {
    constructor(parent2) {
      this.binaryType = "arraybuffer";
      // compatibility with Transport, which sets this
      this.readyState = 0;
      window.addEventListener("message", (event) => {
        if (event.origin !== transport_origin || event.source !== parent2)
          return;
        const data = event.data;
        if (data === void 0 || data.length === void 0 && data.byteLength === void 0)
          return;
        if (data.length === 0) {
          this.readyState = 3;
          this.onclose();
        } else {
          this.onmessage(event);
        }
      }, false);
      window.setTimeout(() => {
        this.readyState = 1;
        this.onopen();
      }, 0);
    }
    // essentially signal handlers: these are assigned to from Transport
    onopen() {
    }
    onclose() {
    }
    onmessage(_event) {
    }
    // same types as the real WebSocket
    send(message) {
      parent.postMessage(message, transport_origin);
    }
    close() {
      this.readyState = 3;
      parent.postMessage("", transport_origin);
      this.onclose();
    }
  };

  // pkg/lib/cockpit/_internal/transport.ts
  var TransportGlobals = class {
    constructor() {
      this.default_transport = null;
      this.reload_after_disconnect = false;
      this.expect_disconnect = false;
      this.init_callback = null;
      this.default_host = null;
      this.process_hints = null;
      this.incoming_filters = [];
    }
  };
  if (!document.cockpit_transport_globals)
    document.cockpit_transport_globals = new TransportGlobals();
  var transport_globals = document.cockpit_transport_globals;
  window.addEventListener("beforeunload", () => {
    transport_globals.expect_disconnect = true;
  }, false);
  function transport_debug(...args) {
    if (window.debugging == "all" || window.debugging?.includes("channel"))
      console.debug(...args);
  }
  var _last_channel, _channel_seed, _ws, _ignore_health_check, _got_message, _check_health_timer, _control_cbs, _message_cbs, _waiting_for_init, _Transport_instances, ready_for_channels_fn, process_init_fn, process_control_fn, process_message_fn;
  var Transport = class extends EventEmitter {
    constructor() {
      super();
      __privateAdd(this, _Transport_instances);
      __privateAdd(this, _last_channel, 0);
      __privateAdd(this, _channel_seed, "");
      __privateAdd(this, _ws);
      __privateAdd(this, _ignore_health_check, false);
      __privateAdd(this, _got_message, false);
      __privateAdd(this, _check_health_timer);
      __privateAdd(this, _control_cbs, {});
      __privateAdd(this, _message_cbs, {});
      __privateAdd(this, _waiting_for_init, true);
      this.application = calculate_application();
      if (window.mock)
        window.mock.last_transport = this;
      if (window.parent !== window && window.name.indexOf("cockpit1:") === 0) {
        __privateSet(this, _ws, new ParentWebSocket(window.parent));
      } else {
        const ws_loc = calculate_url();
        transport_debug("connecting to " + ws_loc);
        __privateSet(this, _ws, new WebSocket(ws_loc, "cockpit1"));
        __privateSet(this, _check_health_timer, window.setInterval(() => {
          if (this.ready && __privateGet(this, _ws))
            __privateGet(this, _ws).send('\n{ "command": "ping" }');
          if (!__privateGet(this, _got_message)) {
            if (__privateGet(this, _ignore_health_check)) {
              console.log("health check failure ignored");
            } else {
              console.log("health check failed");
              this.close({ problem: "timeout" });
            }
          }
          __privateSet(this, _got_message, false);
        }, 3e4));
      }
      this.ready = false;
      __privateGet(this, _ws).onopen = () => {
        if (__privateGet(this, _ws)) {
          if (typeof __privateGet(this, _ws).binaryType !== "undefined")
            __privateGet(this, _ws).binaryType = "arraybuffer";
          __privateGet(this, _ws).send('\n{ "command": "init", "version": 1 }');
        }
      };
      __privateGet(this, _ws).onclose = () => {
        transport_debug("WebSocket onclose");
        __privateSet(this, _ws, null);
        if (transport_globals.reload_after_disconnect) {
          transport_globals.expect_disconnect = true;
          window.location.reload(true);
        }
        this.close();
      };
      __privateGet(this, _ws).onmessage = (event) => this.dispatch_data(event);
    }
    dispatch_data(arg) {
      __privateSet(this, _got_message, true);
      const message = arg.data;
      let channel;
      let control = null;
      let payload = null;
      if (message instanceof ArrayBuffer) {
        const frame = new window.Uint8Array(message);
        const nl = frame.indexOf(10);
        channel = new TextDecoder().decode(frame.subarray(0, nl));
        if (!channel) {
          console.warn("Received invalid binary message without a channel");
          return false;
        }
        payload = frame.subarray(nl + 1);
        transport_debug("recv binary message:", control, payload);
      } else {
        const nl = message.indexOf("\n");
        channel = message.substring(0, nl);
        if (nl == 0) {
          control = JSON.parse(message);
          transport_debug("recv control:", control);
        } else {
          payload = message.substring(nl + 1);
          transport_debug("recv text message:", channel, payload);
        }
      }
      for (const filter of transport_globals.incoming_filters)
        if (filter(message, channel, control) === false)
          return false;
      if (control)
        __privateMethod(this, _Transport_instances, process_control_fn).call(this, control);
      else if (channel && payload)
        __privateMethod(this, _Transport_instances, process_message_fn).call(this, channel, payload);
      return true;
    }
    close(options) {
      if (!options)
        options = { problem: "disconnected" };
      options.command = "close";
      window.clearInterval(__privateGet(this, _check_health_timer));
      const ows = __privateGet(this, _ws);
      __privateSet(this, _ws, null);
      if (ows)
        ows.close();
      if (transport_globals.expect_disconnect)
        return;
      __privateMethod(this, _Transport_instances, ready_for_channels_fn).call(this);
      for (const chan in __privateGet(this, _control_cbs))
        __privateGet(this, _control_cbs)[chan].apply(null, [options]);
    }
    next_channel() {
      __privateWrapper(this, _last_channel)._++;
      return __privateGet(this, _channel_seed) + String(__privateGet(this, _last_channel));
    }
    send_data(data) {
      if (!__privateGet(this, _ws)) {
        return false;
      }
      __privateGet(this, _ws).send(data);
      return true;
    }
    send_message(payload, channel) {
      if (channel)
        transport_debug("send " + channel, payload);
      else
        transport_debug("send control:", payload);
      if (typeof payload !== "string") {
        const body = payload instanceof ArrayBuffer ? new Uint8Array(payload) : payload;
        const header = new TextEncoder().encode(`${channel}
`);
        const output = new Uint8Array(header.length + body.length);
        output.set(header);
        output.set(body, header.length);
        return this.send_data(output.buffer);
      } else {
        return this.send_data(channel.toString() + "\n" + payload);
      }
    }
    send_control(data) {
      if (!__privateGet(this, _ws) && (data.command == "close" || data.command == "kill"))
        return false;
      if (__privateGet(this, _check_health_timer) && data.command == "hint" && data.hint == "ignore_transport_health_check") {
        __privateSet(this, _ignore_health_check, !!data.data);
        return false;
      }
      return this.send_message(JSON.stringify(data), "");
    }
    register(channel, control_cb, message_cb) {
      __privateGet(this, _control_cbs)[channel] = control_cb;
      __privateGet(this, _message_cbs)[channel] = message_cb;
    }
    unregister(channel) {
      delete __privateGet(this, _control_cbs)[channel];
      delete __privateGet(this, _message_cbs)[channel];
    }
  };
  _last_channel = new WeakMap();
  _channel_seed = new WeakMap();
  _ws = new WeakMap();
  _ignore_health_check = new WeakMap();
  _got_message = new WeakMap();
  _check_health_timer = new WeakMap();
  _control_cbs = new WeakMap();
  _message_cbs = new WeakMap();
  _waiting_for_init = new WeakMap();
  _Transport_instances = new WeakSet();
  /* Called when ready for channels to interact */
  ready_for_channels_fn = function() {
    if (!this.ready) {
      this.ready = true;
      this.emit("ready");
    }
  };
  process_init_fn = function(options) {
    if (options.problem) {
      this.close({ problem: options.problem });
      return;
    }
    if (options.version !== 1) {
      console.error("received unsupported version in init message: " + options.version);
      this.close({ problem: "not-supported" });
      return;
    }
    if (options["channel-seed"])
      __privateSet(this, _channel_seed, String(options["channel-seed"]));
    if (typeof options.host === "string")
      transport_globals.default_host = options.host;
    if (transport_globals.init_callback)
      transport_globals.init_callback(options);
    if (__privateGet(this, _waiting_for_init)) {
      __privateSet(this, _waiting_for_init, false);
      __privateMethod(this, _Transport_instances, ready_for_channels_fn).call(this);
    }
  };
  process_control_fn = function(data) {
    const channel = data.channel;
    if (data.command == "init") {
      __privateMethod(this, _Transport_instances, process_init_fn).call(this, data);
    } else if (__privateGet(this, _waiting_for_init)) {
      __privateSet(this, _waiting_for_init, false);
      if (data.command != "close" || channel) {
        console.error("received message before init: ", data.command);
        data = { problem: "protocol-error" };
      }
      this.close(data);
    } else if (data.command == "ping") {
      data.command = "pong";
      this.send_control(data);
    } else if (data.command == "pong") {
    } else if (data.command == "hint") {
      if (transport_globals.process_hints)
        transport_globals.process_hints(data);
    } else if (typeof channel === "string") {
      const func = __privateGet(this, _control_cbs)[channel];
      if (func)
        func(data);
    }
  };
  process_message_fn = function(channel, payload) {
    const func = __privateGet(this, _message_cbs)[channel];
    if (func)
      func(payload);
  };
  function ensure_transport(callback) {
    if (!transport_globals.default_transport)
      transport_globals.default_transport = new Transport();
    const transport = transport_globals.default_transport;
    if (transport.ready) {
      callback(transport);
    } else {
      transport.on("ready", () => {
        callback(transport);
      });
    }
  }
  window.addEventListener("unload", () => {
    if (transport_globals.default_transport)
      transport_globals.default_transport.close();
  });

  // pkg/lib/cockpit/_internal/channel.js
  function Channel(options) {
    const self = this;
    event_mixin(self, {});
    let transport;
    let ready = null;
    let closed = null;
    let waiting = null;
    let received_done = false;
    let sent_done = false;
    let id = null;
    const binary = options.binary === true;
    const queue = [];
    self.valid = true;
    self.options = options;
    self.binary = binary;
    self.id = id;
    function on_message(payload) {
      if (received_done) {
        console.warn("received message after done");
        self.close("protocol-error");
      } else {
        self.dispatchEvent("message", payload);
      }
    }
    function on_close(data) {
      closed = data;
      self.valid = false;
      if (transport && id)
        transport.unregister(id);
      if (closed.message && !options.err)
        console.warn(closed.message);
      self.dispatchEvent("close", closed);
      if (waiting)
        waiting.resolve(closed);
    }
    function on_ready(data) {
      ready = data;
      self.dispatchEvent("ready", ready);
    }
    function on_control(data) {
      if (data.command == "close") {
        on_close(data);
        return;
      } else if (data.command == "ready") {
        on_ready(data);
      }
      const done = data.command === "done";
      if (done && received_done) {
        console.warn("received two done commands on channel");
        self.close("protocol-error");
      } else {
        if (done)
          received_done = true;
        self.dispatchEvent("control", data);
      }
    }
    function send_payload(payload) {
      if (!binary) {
        if (typeof payload !== "string")
          payload = String(payload);
      }
      transport.send_message(payload, id);
    }
    ensure_transport(function(trans) {
      transport = trans;
      if (closed)
        return;
      id = transport.next_channel();
      self.id = id;
      transport.register(id, on_control, on_message);
      const command = {};
      for (const i in options)
        if (i !== "binary")
          command[i] = options[i];
      if (binary)
        command.binary = "raw";
      command.command = "open";
      command.channel = id;
      if (!command.host) {
        if (transport_globals.default_host)
          command.host = transport_globals.default_host;
      }
      command["flow-control"] = true;
      transport.send_control(command);
      while (queue.length > 0) {
        const item = queue.shift();
        if (item[0]) {
          item[1].channel = id;
          transport.send_control(item[1]);
        } else {
          send_payload(item[1]);
        }
      }
    });
    self.send = function send(message) {
      if (closed)
        console.warn("sending message on closed channel");
      else if (sent_done)
        console.warn("sending message after done");
      else if (!transport)
        queue.push([false, message]);
      else
        send_payload(message);
    };
    self.control = function control(options2) {
      options2 = options2 || {};
      if (!options2.command)
        options2.command = "options";
      if (options2.command === "done")
        sent_done = true;
      options2.channel = id;
      if (!transport)
        queue.push([true, options2]);
      else
        transport.send_control(options2);
    };
    self.wait = function wait(callback) {
      if (!waiting) {
        waiting = new Deferred();
        if (closed) {
          waiting.reject(closed);
        } else if (ready) {
          waiting.resolve(ready);
        } else {
          self.addEventListener("ready", function(event, data) {
            waiting.resolve(data);
          });
          self.addEventListener("close", function(event, data) {
            waiting.reject(data);
          });
        }
      }
      const promise = waiting.promise;
      if (callback)
        promise.then(callback, callback);
      return promise;
    };
    self.close = function close(options2) {
      if (closed)
        return;
      if (!options2)
        options2 = {};
      else if (typeof options2 == "string")
        options2 = { problem: options2 };
      options2.command = "close";
      options2.channel = id;
      if (!transport)
        queue.push([true, options2]);
      else
        transport.send_control(options2);
      on_close(options2);
    };
    self.buffer = function buffer(callback) {
      const buffers = [];
      buffers.callback = callback;
      buffers.squash = function squash() {
        return join_data(buffers, binary);
      };
      function on_message2(event, data) {
        buffers.push(data);
        if (buffers.callback) {
          const block = join_data(buffers, binary);
          if (block.length > 0) {
            const consumed = buffers.callback.call(self, block);
            if (typeof consumed !== "number" || consumed === block.length) {
              buffers.length = 0;
            } else if (consumed === 0) {
              buffers.length = 1;
              buffers[0] = block;
            } else if (consumed !== 0) {
              buffers.length = 1;
              if (block.subarray)
                buffers[0] = block.subarray(consumed);
              else if (block.substring)
                buffers[0] = block.substring(consumed);
              else
                buffers[0] = block.slice(consumed);
            }
          }
        }
      }
      function on_close2() {
        self.removeEventListener("message", on_message2);
        self.removeEventListener("close", on_close2);
      }
      self.addEventListener("message", on_message2);
      self.addEventListener("close", on_close2);
      return buffers;
    };
    self.toString = function toString() {
      const host = options.host || "localhost";
      return "[Channel " + (self.valid ? id : "<invalid>") + " -> " + host + "]";
    };
  }

  // pkg/lib/cockpit/_internal/location.ts
  var _hash_changed, _Location_instances, resolve_path_dots_fn, href_for_go_or_replace_fn, decode_path_fn;
  var _Location = class _Location {
    constructor() {
      __privateAdd(this, _Location_instances);
      __privateAdd(this, _hash_changed, false);
      const application = calculate_application();
      this.url_root = url_root || "";
      if (window.mock?.url_root)
        this.url_root = window.mock.url_root;
      if (application.indexOf("cockpit+=") === 0) {
        if (this.url_root)
          this.url_root += "/";
        this.url_root = this.url_root + application.replace("cockpit+", "");
      }
      this.href = window.location.hash.slice(1);
      this.options = {};
      this.path = this.decode(this.href, this.options);
    }
    encode(path, options, with_root = false) {
      if (typeof path == "string")
        path = __privateMethod(this, _Location_instances, decode_path_fn).call(this, path);
      let href = "/" + path.map(encodeURIComponent).join("/");
      if (with_root && this.url_root && href.indexOf("/" + this.url_root + "/") !== 0)
        href = "/" + this.url_root + href;
      href = href.replaceAll("%40", "@");
      href = href.replaceAll("%3D", "=");
      href = href.replaceAll("%2B", "+");
      href = href.replaceAll("%23", "#");
      const query = [];
      if (options) {
        for (const opt in options) {
          let value = options[opt];
          if (!Array.isArray(value))
            value = [value];
          value.forEach(function(v) {
            query.push(encodeURIComponent(opt) + "=" + encodeURIComponent(v));
          });
        }
        if (query.length > 0)
          href += "?" + query.join("&");
      }
      return href;
    }
    // NOTE: The options argument is modified in place
    decode(href, options) {
      if (href[0] == "#")
        href = href.substring(1);
      const pos = href.indexOf("?");
      const first = pos === -1 ? href : href.substring(0, pos);
      const path = __privateMethod(this, _Location_instances, decode_path_fn).call(this, first);
      if (pos !== -1 && options) {
        href.substring(pos + 1).split("&").forEach(function(opt) {
          const parts = opt.split("=");
          const name = decodeURIComponent(parts[0]);
          const value = decodeURIComponent(parts[1]);
          if (options[name]) {
            let last = options[name];
            if (!Array.isArray(last))
              last = options[name] = [last];
            last.push(value);
          } else {
            options[name] = value;
          }
        });
      }
      return path;
    }
    replace(path, options) {
      if (__privateGet(this, _hash_changed))
        return;
      const href = __privateMethod(this, _Location_instances, href_for_go_or_replace_fn).call(this, path, options);
      window.location.replace(window.location.pathname + "#" + href);
    }
    go(path, options) {
      if (__privateGet(this, _hash_changed))
        return;
      const href = __privateMethod(this, _Location_instances, href_for_go_or_replace_fn).call(this, path, options);
      window.location.hash = "#" + href;
    }
    invalidate() {
      __privateSet(this, _hash_changed, true);
    }
    toString() {
      return this.href;
    }
  };
  _hash_changed = new WeakMap();
  _Location_instances = new WeakSet();
  resolve_path_dots_fn = function(parts) {
    const out = [];
    for (let i = 0; i < parts.length; i++) {
      const part = parts[i];
      if (part === "" || part == ".") {
        continue;
      } else if (part == "..") {
        if (out.length === 0)
          return [];
        out.pop();
      } else {
        out.push(part);
      }
    }
    return out;
  };
  href_for_go_or_replace_fn = function(path, options) {
    options = options || {};
    if (typeof path === "string") {
      return this.encode(this.decode(path, options), options);
    } else if (path instanceof _Location) {
      return path.href;
    } else {
      return this.encode(path, options);
    }
  };
  decode_path_fn = function(input) {
    let result, i;
    let pre_parts = [];
    const parts = input.split("/").map(decodeURIComponent);
    if (this.url_root)
      pre_parts = this.url_root.split("/").map(decodeURIComponent);
    if (input && input[0] !== "/" && this.path !== void 0) {
      result = [...this.path];
      result.pop();
      result = result.concat(parts);
    } else {
      result = parts;
    }
    result = __privateMethod(this, _Location_instances, resolve_path_dots_fn).call(this, result);
    for (i = 0; i < pre_parts.length; i++) {
      if (pre_parts[i] !== result[i])
        break;
    }
    if (i == pre_parts.length)
      result.splice(0, pre_parts.length);
    return result;
  };
  var Location = _Location;

  // pkg/lib/cockpit/channel.ts
  var _transport, _received, _queue, _sent_done, _Channel_instances, on_control_fn;
  var Channel2 = class extends EventEmitter {
    /**
     * Open a new channel to the bridge.
     *
     * @options: The options for the channel.  A payload type must be specified.
     */
    constructor(options) {
      super();
      __privateAdd(this, _Channel_instances);
      this.id = null;
      __privateAdd(this, _transport, null);
      __privateAdd(this, _received, {});
      __privateAdd(this, _queue, []);
      __privateAdd(this, _sent_done, false);
      this.options = { ...options };
      this.binary = options?.binary === true;
      ensure_transport((transport) => {
        if (__privateGet(this, _received).close)
          return;
        __privateSet(this, _transport, transport);
        this.id = transport.next_channel();
        transport.register(
          this.id,
          (control) => {
            if (typeof control.command !== "string") {
              console.error("Received control message without command", this.id, this.options, control);
            } else {
              __privateMethod(this, _Channel_instances, on_control_fn).call(this, control);
            }
          },
          (data) => {
            if (this.binary && typeof data === "string") {
              console.error("Text message received on binary channel", this.id, this.options, data);
            } else if (!this.binary && typeof data !== "string") {
              console.error("Binary message received on text channel", this.id, this.options, data);
            } else {
              this.emit("data", data);
            }
          }
        );
        const command = { ...this.options };
        if (!command.host && transport_globals.default_host) {
          command.host = transport_globals.default_host;
        }
        if (this.binary) {
          command.binary = "raw";
        } else {
          delete command.binary;
        }
        transport.send_control({ ...command, command: "open", channel: this.id, "flow-control": true });
        for (const [is_control, message] of __privateGet(this, _queue)) {
          if (is_control) {
            transport.send_control({ ...message, channel: this.id });
          } else {
            transport.send_message(message, this.id);
          }
        }
        __privateSet(this, _queue, []);
      });
    }
    /**
     * Sends a payload frame.
     *
     * You may not call this after you've sent a 'done' control message or
     * after the channel has been closed.  This implies that you need to
     * register a 'close' event handler, and stop sending data after it's
     * called.
     *
     * @message the payload to send, either a string or a Uint8Array.
     */
    send_data(message) {
      if (__privateGet(this, _sent_done)) {
        console.error("sending data after .done()", this.id, this.options, message);
      } else if (__privateGet(this, _received).close) {
        console.error("sending data after close", this.id, this.options, message);
      } else if (__privateGet(this, _transport) && this.id) {
        __privateGet(this, _transport).send_message(message, this.id);
      } else {
        __privateGet(this, _queue).push([false, message]);
      }
    }
    /**
     * Sends a control message on the channel.
     *
     * You may not call this after the channel is closed.  This implies that
     * you need to register a 'close' event handler, and stop sending data
     * after it's called.
     *
     * @options: the message to send.  A command must be specified.
     */
    send_control(options) {
      if (__privateGet(this, _received).close) {
        console.error("sending control after close", this.id, this.options, options);
        return;
      }
      if (options.command === "close") {
        __privateMethod(this, _Channel_instances, on_control_fn).call(this, options);
      } else if (options.command === "done") {
        __privateSet(this, _sent_done, true);
      }
      if (__privateGet(this, _transport) && this.id) {
        __privateGet(this, _transport).send_control({ ...options, channel: this.id });
      } else {
        __privateGet(this, _queue).push([true, options]);
      }
    }
    /**
     * Sends a done control message on the channel.  This is something like
     * EOF: it means that you won't send any more data using `.send_data()`.
     *
     * @options: optional extra arguments for the message.
     */
    done(options) {
      this.send_control({ ...options, command: "done" });
    }
    /**
     * Closes the channel.
     *
     * This means that you're completely finished with the channel.  Any
     * underlying resources will be freed as soon as possible.  When you call
     * this you'll receive a 'close' signal (synchronously) and then nothing
     * else.
     *
     * @problem: a problem code.  If this is unset it implies something like a
     * "successful" close.  Otherwise, it indicates an error.
     * @options: the bridge will ignore this, but it will be thrown as the
     * result of any pending wait() operations and passed to the 'close' signal
     * handler, so you can use it to communicate with your own code.
     */
    close(problem, options) {
      if (!__privateGet(this, _received).close) {
        this.send_control({ ...options, ...problem && { problem }, command: "close" });
      }
    }
    /**
     * Waits for the result of the channel open request.
     *
     * @return: the content of the ready message, on success
     * @throws: the content of the close message, on fail
     */
    wait() {
      return new Promise((resolve, reject) => {
        if (__privateGet(this, _received).ready) {
          resolve(__privateGet(this, _received).ready);
        } else if (__privateGet(this, _received).close) {
          reject(__privateGet(this, _received).close);
        } else {
          this.on("ready", resolve);
          this.on("close", reject);
        }
      });
    }
    /**
     * Provides a text description of the channel.
     */
    toString() {
      const state = !this.id && "waiting for transport" || __privateGet(this, _received).close?.problem && `${this.id} error ${__privateGet(this, _received).close.problem}` || __privateGet(this, _received).close && `${this.id} closed` || __privateGet(this, _received).ready && `${this.id} opened` || `${this.id} waiting for open`;
      const host = this.options?.host || "localhost";
      return `[Channel ${state} -> ${this.options?.payload}@${host}]`;
    }
  };
  _transport = new WeakMap();
  _received = new WeakMap();
  _queue = new WeakMap();
  _sent_done = new WeakMap();
  _Channel_instances = new WeakSet();
  on_control_fn = function(control) {
    const command = control.command;
    if (command === "ready" || command === "close" || command === "done") {
      if (__privateGet(this, _received)[command]) {
        console.error("received duplicate control message", this.id, this.options, control);
        return;
      }
      __privateGet(this, _received)[command] = control;
      this.emit(command, control);
    } else {
      this.emit("control", control);
    }
    if (command === "close") {
      if (__privateGet(this, _transport) && this.id)
        __privateGet(this, _transport).unregister(this.id);
      if (control.message && !this.options?.err)
        console.warn("channel error", control.message, this.id, this.options);
    }
  };

  // pkg/lib/cockpit/fsinfo.ts
  function is_json_dict(value) {
    return value?.constructor === Object;
  }
  function json_merge(current, patch) {
    if (is_json_dict(patch)) {
      const updated = is_json_dict(current) ? { ...current } : {};
      for (const [key, value] of Object.entries(patch)) {
        if (value === null) {
          delete updated[key];
        } else {
          updated[key] = json_merge(updated[key], value);
        }
      }
      return updated;
    } else {
      return patch;
    }
  }
  var FsInfoClient = class extends EventEmitter {
    constructor(path, attrs, options) {
      super();
      this.state = { loading: true };
      this.partial_state = null;
      this.channel = new Channel2({
        payload: "fsinfo",
        path,
        attrs,
        watch: true,
        ...options
      });
      this.channel.on("data", (payload) => {
        this.partial_state = json_merge(this.partial_state, JSON.parse(payload));
        if (is_json_dict(this.partial_state) && !this.partial_state.partial) {
          this.state = { ...this.partial_state };
          this.emit("change", this.state);
        }
      });
      this.channel.on("close", (message) => {
        this.emit("close", message);
      });
    }
    close() {
      this.channel.close();
    }
    static entry(info, name) {
      return info.entries?.[name] ?? null;
    }
    static target(info, name) {
      const entries = info.entries ?? {};
      const targets = info.targets ?? {};
      let entry = entries[name] ?? null;
      for (let i = 0; i < 40; i++) {
        const target = entry?.target;
        if (!target)
          return entry;
        if (target === ".")
          return info;
        entry = entries[target] ?? targets[target] ?? null;
      }
      return null;
    }
  };

  // pkg/lib/cockpit/_internal/info.ts
  function fetch_info() {
    const channel = new Channel2({ payload: "info" });
    return new Promise((resolve, reject) => {
      channel.on("data", (data) => {
        resolve(JSON.parse(data));
        channel.close();
      });
      channel.on("close", (msg) => {
        reject(msg);
      });
    });
  }

  // pkg/lib/cockpit.js
  function factory() {
    const cockpit2 = {};
    event_mixin(cockpit2, {});
    cockpit2.init = async () => {
      await new Promise((resolve) => ensure_transport(resolve));
      Object.assign(cockpit2.info, await fetch_info());
    };
    cockpit2.channel = function channel(options) {
      return new Channel(options);
    };
    cockpit2.event_target = function event_target(obj) {
      event_mixin(obj, {});
      return obj;
    };
    cockpit2.extend = Object.assign;
    cockpit2.manifests = {};
    cockpit2.base64_encode = base64_encode;
    cockpit2.base64_decode = base64_decode;
    cockpit2.kill = function kill(host, group) {
      const options = {};
      if (host)
        options.host = host;
      if (group)
        options.group = group;
      cockpit2.transport.control("kill", options);
    };
    cockpit2.hint = function hint(name, options) {
      if (!transport_globals.default_transport)
        return;
      if (!options)
        options = transport_globals.default_host;
      if (typeof options == "string")
        options = { host: options };
      options.hint = name;
      cockpit2.transport.control("hint", options);
    };
    cockpit2.transport = {
      wait: ensure_transport,
      inject: function inject(message, out) {
        if (!transport_globals.default_transport)
          return false;
        if (out === void 0 || out)
          return transport_globals.default_transport.send_data(message);
        else
          return transport_globals.default_transport.dispatch_data({ data: message });
      },
      filter: function filter(callback, out) {
        if (out) {
          console.error("'out' filters are no longer supported");
        } else {
          transport_globals.incoming_filters.push(callback);
        }
      },
      close: function close(problem) {
        if (transport_globals.default_transport)
          transport_globals.default_transport.close(problem ? { problem } : void 0);
        transport_globals.default_transport = null;
        this.options = {};
      },
      origin: transport_origin,
      options: {},
      uri: calculate_url,
      control: function(command, options) {
        options = { ...options, command };
        ensure_transport(function(transport) {
          transport.send_control(options);
        });
      },
      application: function() {
        if (!transport_globals.default_transport || window.mock)
          return calculate_application();
        return transport_globals.default_transport.application;
      }
    };
    cockpit2.resolve = function resolve(result) {
      console.warn("cockpit.resolve() is deprecated. Use Promise.resolve()");
      return Promise.resolve(result);
    };
    cockpit2.reject = function reject(ex) {
      console.warn("cockpit.reject() is deprecated. Use Promise.reject()");
      return Promise.reject(ex);
    };
    cockpit2.defer = function() {
      return new Deferred();
    };
    const fmt_re = /\$\{([^}]+)\}|\$([a-zA-Z0-9_]+)/g;
    cockpit2.format = function format(fmt, args) {
      if (arguments.length != 2 || !is_object(args) || args === null)
        args = Array.prototype.slice.call(arguments, 1);
      function replace(m, x, y) {
        const value = args[x || y];
        if (value === 0)
          return "0";
        return value || "";
      }
      return fmt.replace(fmt_re, replace);
    };
    cockpit2.format_number = function format_number(number, precision) {
      if (precision === void 0)
        precision = 3;
      const lang = cockpit2.language === void 0 ? void 0 : cockpit2.language.replace("_", "-");
      const smallestValue = 10 ** -precision;
      if (!number && number !== 0)
        return "";
      else if (number % 1 === 0)
        return number.toString();
      else if (number > 0 && number <= smallestValue)
        return smallestValue.toLocaleString(lang);
      else if (number < 0 && number >= -smallestValue)
        return (-smallestValue).toLocaleString(lang);
      else if (number > 999 || number < -999)
        return number.toFixed(0);
      else
        return number.toLocaleString(lang, {
          maximumSignificantDigits: precision,
          minimumSignificantDigits: precision
        });
    };
    let deprecated_format_warned = false;
    function format_units(suffixes, number, second_arg, third_arg) {
      let options = second_arg;
      let factor = options?.base2 ? 1024 : 1e3;
      if (third_arg || second_arg && !is_object(second_arg)) {
        if (!deprecated_format_warned) {
          console.warn(`cockpit.format_{bytes,bits}[_per_sec](..., ${second_arg}, ${third_arg}) is deprecated.`);
          deprecated_format_warned = true;
        }
        factor = second_arg || 1e3;
        options = third_arg;
        if (!is_object(options))
          options = { separate: options };
      }
      let suffix = null;
      if (!number && number !== 0) {
        suffix = null;
      } else if (typeof factor === "string") {
        const keys = [];
        for (const key in suffixes)
          keys.push(key);
        keys.sort().reverse();
        for (let y = 0; y < keys.length; y++) {
          for (let x = 0; x < suffixes[keys[y]].length; x++) {
            if (factor == suffixes[keys[y]][x]) {
              number = number / Math.pow(keys[y], x);
              suffix = factor;
              break;
            }
          }
          if (suffix)
            break;
        }
      } else if (factor in suffixes) {
        let divisor = 1;
        for (let i = 0; i < suffixes[factor].length; i++) {
          const quotient = number / divisor;
          if (quotient < factor) {
            number = quotient;
            suffix = suffixes[factor][i];
            break;
          }
          divisor *= factor;
        }
      }
      const string_representation = cockpit2.format_number(number, options?.precision);
      let ret;
      if (string_representation && suffix)
        ret = [string_representation, suffix];
      else
        ret = [string_representation];
      if (!options?.separate)
        ret = ret.join(" ");
      return ret;
    }
    const byte_suffixes = {
      1e3: ["B", "kB", "MB", "GB", "TB", "PB", "EB", "ZB"],
      1024: ["B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB"]
    };
    cockpit2.format_bytes = function format_bytes(number, ...args) {
      return format_units(byte_suffixes, number, ...args);
    };
    const byte_sec_suffixes = {
      1e3: ["B/s", "kB/s", "MB/s", "GB/s", "TB/s", "PB/s", "EB/s", "ZB/s"],
      1024: ["B/s", "KiB/s", "MiB/s", "GiB/s", "TiB/s", "PiB/s", "EiB/s", "ZiB/s"]
    };
    cockpit2.format_bytes_per_sec = function format_bytes_per_sec(number, ...args) {
      return format_units(byte_sec_suffixes, number, ...args);
    };
    const bit_suffixes = {
      1e3: ["bps", "Kbps", "Mbps", "Gbps", "Tbps", "Pbps", "Ebps", "Zbps"]
    };
    cockpit2.format_bits_per_sec = function format_bits_per_sec(number, ...args) {
      return format_units(bit_suffixes, number, ...args);
    };
    function StorageHelper(storageName) {
      const self = this;
      let storage;
      try {
        storage = window[storageName];
      } catch (e) {
      }
      self.prefixedKey = function(key) {
        return cockpit2.transport.application() + ":" + key;
      };
      self.getItem = function(key, both) {
        let value = storage.getItem(self.prefixedKey(key));
        if (!value && both)
          value = storage.getItem(key);
        return value;
      };
      self.setItem = function(key, value, both) {
        storage.setItem(self.prefixedKey(key), value);
        if (both)
          storage.setItem(key, value);
      };
      self.removeItem = function(key, both) {
        storage.removeItem(self.prefixedKey(key));
        if (both)
          storage.removeItem(key);
      };
      self.clear = function(full) {
        let i = 0;
        while (i < storage.length) {
          const k = storage.key(i);
          if (full && k.indexOf("cockpit") !== 0)
            storage.removeItem(k);
          else if (k.indexOf(cockpit2.transport.application()) === 0)
            storage.removeItem(k);
          else
            i++;
        }
      };
    }
    cockpit2.localStorage = new StorageHelper("localStorage");
    cockpit2.sessionStorage = new StorageHelper("sessionStorage");
    function lookup_storage(win) {
      let storage;
      if (win.parent && win.parent !== win)
        storage = lookup_storage(win.parent);
      if (!storage) {
        try {
          storage = win["cv1-storage"];
          if (!storage)
            win["cv1-storage"] = storage = {};
        } catch (ex) {
        }
      }
      return storage;
    }
    function StorageCache(org_key, provider, consumer) {
      const self = this;
      const key = cockpit2.transport.application() + ":" + org_key;
      const trigger = window.sessionStorage;
      let last;
      const storage = lookup_storage(window);
      let claimed = false;
      let source;
      function callback() {
        if (storage[key] !== void 0) {
          const value = storage[key];
          window.setTimeout(function() {
            if (consumer(value, org_key) === false)
              self.close();
          });
        }
      }
      function result(value) {
        if (source && !claimed)
          claimed = true;
        if (!claimed)
          return;
        const version = Math.floor(Math.random() * 1e7) + 1;
        const ev = document.createEvent("StorageEvent");
        ev.initStorageEvent(
          "storage",
          false,
          false,
          key,
          null,
          version,
          window.location,
          trigger
        );
        storage[key] = value;
        trigger.setItem(key, version);
        ev.self = self;
        window.dispatchEvent(ev);
      }
      self.claim = function claim() {
        if (source)
          return;
        const claiming = { close: function() {
        } };
        source = claiming;
        const changed2 = provider(result, org_key);
        if (source === claiming)
          source = changed2;
        else
          changed2.close();
      };
      function unclaim() {
        if (source?.close)
          source.close();
        source = null;
        if (!claimed)
          return;
        claimed = false;
        let current_value = trigger.getItem(key);
        if (current_value)
          current_value = parseInt(current_value, 10);
        else
          current_value = null;
        if (last && last === current_value) {
          const ev = document.createEvent("StorageEvent");
          const version = trigger[key];
          ev.initStorageEvent(
            "storage",
            false,
            false,
            key,
            version,
            null,
            window.location,
            trigger
          );
          delete storage[key];
          trigger.removeItem(key);
          ev.self = self;
          window.dispatchEvent(ev);
        }
      }
      function changed(event) {
        if (event.key !== key)
          return;
        if (event.self !== self) {
          if (!event.newValue && !claimed) {
            self.claim();
            return;
          }
        } else if (claimed && !event.oldValue && event.newValue !== trigger.getItem(key)) {
          unclaim();
        }
        let new_value = null;
        if (event.newValue)
          new_value = parseInt(event.newValue, 10);
        if (last !== new_value) {
          last = new_value;
          callback();
        }
      }
      self.close = function() {
        window.removeEventListener("storage", changed, true);
        unclaim();
      };
      window.addEventListener("storage", changed, true);
      window.addEventListener("beforeunload", function() {
        self.close();
      });
      window.addEventListener("unload", function() {
        self.close();
      });
      if (trigger.getItem(key))
        callback();
      else
        self.claim();
    }
    cockpit2.cache = function cache(key, provider, consumer) {
      return new StorageCache(key, provider, consumer);
    };
    function SeriesSink(interval, identifier, fetch_callback) {
      const self = this;
      self.interval = interval;
      self.limit = identifier ? 64 * 1024 : 1024;
      const index = setup_index(identifier);
      let count = 0;
      let head = null;
      let tail = null;
      function setup_index(id) {
        if (!id)
          return [];
        const storage = lookup_storage(window);
        let index2 = storage[id];
        if (!index2)
          storage[id] = index2 = [];
        return index2;
      }
      function search(idx, beg) {
        let low = 0;
        let high = idx.length - 1;
        while (low <= high) {
          const mid = (low + high) / 2 | 0;
          const val = idx[mid].beg;
          if (val < beg)
            low = mid + 1;
          else if (val > beg)
            high = mid - 1;
          else
            return mid;
        }
        return low;
      }
      function fetch(beg, end, for_walking) {
        if (fetch_callback) {
          if (!for_walking) {
            stash(beg, new Array(end - beg), {});
          }
          fetch_callback(beg, end, for_walking);
        }
      }
      self.load = function load(beg, end, for_walking) {
        if (end <= beg)
          return;
        const at = search(index, beg);
        const len = index.length;
        let last = beg;
        const fetches = [];
        for (let i = at > 0 ? at - 1 : at; i < len; i++) {
          const entry = index[i];
          const en = entry.items.length;
          if (!en)
            continue;
          const eb = entry.beg;
          const b = Math.max(eb, beg);
          const e = Math.min(eb + en, end);
          if (b < e) {
            if (b > last)
              fetches.push([last, b]);
            process(b, entry.items.slice(b - eb, e - eb), entry.mapping);
            last = e;
          } else if (i >= at) {
            break;
          }
        }
        for (let i = 0; i < fetches.length; i++)
          fetch(fetches[i][0], fetches[i][1], for_walking);
        if (last != end)
          fetch(last, end, for_walking);
      };
      function stash(beg, items, mapping) {
        if (!items.length)
          return;
        let at = search(index, beg);
        const end = beg + items.length;
        const len = index.length;
        let i;
        for (i = at > 0 ? at - 1 : at; i < len; i++) {
          const entry2 = index[i];
          const en = entry2.items.length;
          if (!en)
            continue;
          const eb = entry2.beg;
          const b = Math.max(eb, beg);
          const e = Math.min(eb + en, end);
          if (b < e) {
            const num = e - b;
            entry2.items.splice(b - eb, num);
            count -= num;
            if (b - eb === 0)
              entry2.beg += e - eb;
          } else if (i >= at) {
            break;
          }
        }
        const entry = { beg, items, mapping };
        if (!head)
          head = entry;
        if (tail)
          tail.next = entry;
        tail = entry;
        count += items.length;
        index.splice(at, 0, entry);
        for (at--; at <= i; at++) {
          const entry2 = index[at];
          if (entry2 && !entry2.items.length) {
            index.splice(at, 1);
            at--;
          }
        }
        while (head && count > self.limit) {
          count -= head.items.length;
          head.items = [];
          head.mapping = null;
          head = head.next || null;
        }
        const newlen = index.length;
        for (i = 0; i < newlen; i++) {
          if (index[i].items.length > 0)
            break;
        }
        index.splice(0, i);
      }
      const registered = {};
      self._register = function _register(grid, id) {
        if (grid.interval != interval)
          throw Error("mismatched metric interval between grid and sink");
        let gdata = registered[id];
        if (!gdata) {
          gdata = registered[id] = { grid, links: [] };
          gdata.links.remove = function remove() {
            delete registered[id];
          };
        }
        return gdata.links;
      };
      function process(beg, items, mapping) {
        const end = beg + items.length;
        for (const id in registered) {
          const gdata = registered[id];
          const grid = gdata.grid;
          const b = Math.max(beg, grid.beg);
          const e = Math.min(end, grid.end);
          if (b < e) {
            const f = b - beg;
            const t = b - grid.beg;
            const n = e - b;
            for (let i = 0; i < n; i++) {
              const klen = gdata.links.length;
              for (let k = 0; k < klen; k++) {
                const path = gdata.links[k][0];
                const row = gdata.links[k][1];
                let data = items[f + i];
                let map = mapping;
                const jlen = path.length;
                for (let j = 0; data !== void 0 && j < jlen; j++) {
                  if (!data) {
                    data = void 0;
                  } else if (map !== void 0 && map !== null) {
                    map = map[path[j]];
                    if (map)
                      data = data[map[""]];
                    else
                      data = data[path[j]];
                  } else {
                    data = data[path[j]];
                  }
                }
                row[t + i] = data;
              }
            }
            grid.notify(t, n);
          }
        }
      }
      self.input = function input(beg, items, mapping) {
        process(beg, items, mapping);
        stash(beg, items, mapping);
      };
      self.close = function() {
        for (const id in registered) {
          const grid = registered[id];
          if (grid?.grid)
            grid.grid.remove_sink(self);
        }
      };
    }
    cockpit2.series = function series(interval, cache, fetch) {
      return new SeriesSink(interval, cache, fetch);
    };
    let unique = 1;
    function SeriesGrid(interval, beg, end) {
      const self = this;
      event_mixin(self, {});
      const rows = [];
      self.interval = interval;
      self.beg = 0;
      self.end = 0;
      const callbacks = [];
      const sinks = [];
      let suppress = 0;
      const id = "g1-" + unique;
      unique += 1;
      let walking = null;
      let offset = null;
      self.notify = function notify(x, n) {
        if (suppress)
          return;
        if (x + n > self.end - self.beg)
          n = self.end - self.beg - x;
        if (n <= 0)
          return;
        const jlen = callbacks.length;
        for (let j = 0; j < jlen; j++) {
          const callback = callbacks[j][0];
          const row = callbacks[j][1];
          callback.call(self, row, x, n);
        }
        self.dispatchEvent("notify", x, n);
      };
      self.add = function add() {
        const row = [];
        rows.push(row);
        if (is_object(arguments[0])) {
          const sink = arguments[0].series || arguments[0];
          let path = arguments[1];
          if (!path)
            path = [];
          else if (typeof path === "string")
            path = path.split(".");
          const links = sink._register(self, id);
          if (!links.length)
            sinks.push({ sink, links });
          links.push([path, row]);
        } else if (is_function(arguments[0])) {
          const cb = [arguments[0], row];
          if (arguments[1] === true)
            callbacks.unshift(cb);
          else
            callbacks.push(cb);
        } else if (arguments.length !== 0) {
          throw Error("invalid args to grid.add()");
        }
        return row;
      };
      self.remove = function remove(row) {
        let ilen = sinks.length;
        for (let i = 0; i < ilen; i++) {
          const jlen = sinks[i].links.length;
          for (let j = 0; j < jlen; j++) {
            if (sinks[i].links[j][1] === row) {
              sinks[i].links.splice(j, 1);
              break;
            }
          }
        }
        ilen = rows.length;
        for (let i = 0; i < ilen; i++) {
          if (rows[i] === row) {
            rows.splice(i, 1);
            break;
          }
        }
      };
      self.remove_sink = function remove_sink(sink) {
        const len = sinks.length;
        for (let i = 0; i < len; i++) {
          if (sinks[i].sink === sink) {
            sinks[i].links.remove();
            sinks.splice(i, 1);
            break;
          }
        }
      };
      self.sync = function sync(for_walking) {
        suppress++;
        const len = sinks.length;
        for (let i = 0; i < len; i++) {
          const sink = sinks[i].sink;
          sink.load(self.beg, self.end, for_walking);
        }
        suppress--;
        self.notify(0, self.end - self.beg);
      };
      function move_internal(beg2, end2, for_walking) {
        if (end2 === void 0)
          end2 = beg2 + (self.end - self.beg);
        if (end2 < beg2)
          beg2 = end2;
        self.beg = beg2;
        self.end = end2;
        if (!rows.length)
          return;
        rows.forEach(function(row) {
          row.length = 0;
        });
        self.sync(for_walking);
      }
      function stop_walking() {
        window.clearInterval(walking);
        walking = null;
        offset = null;
      }
      function is_negative(n) {
        return ((n = +n) || 1 / n) < 0;
      }
      self.move = function move(beg2, end2) {
        stop_walking();
        let now = null;
        if (beg2 === void 0) {
          beg2 = 0;
        } else if (is_negative(beg2)) {
          now = Date.now();
          beg2 = Math.floor(now / self.interval) + beg2;
        }
        if (end2 !== void 0 && is_negative(end2)) {
          if (now === null)
            now = Date.now();
          end2 = Math.floor(now / self.interval) + end2;
        }
        move_internal(beg2, end2, false);
      };
      self.walk = function walk() {
        const start = Date.now();
        if (self.interval > 2e9)
          return;
        stop_walking();
        offset = start - self.beg * self.interval;
        walking = window.setInterval(function() {
          const now = Date.now();
          move_internal(Math.floor((now - offset) / self.interval), void 0, true);
        }, self.interval);
      };
      self.close = function close() {
        stop_walking();
        while (sinks.length)
          sinks.pop().links.remove();
      };
      self.move(beg, end);
    }
    cockpit2.grid = function grid(interval, beg, end) {
      return new SeriesGrid(interval, beg, end);
    };
    function BasicError(problem, message) {
      this.problem = problem;
      this.message = message || cockpit2.message(problem);
      this.toString = function() {
        return this.message;
      };
    }
    cockpit2.logout = function logout(reload, reason) {
      cockpit2.sessionStorage.clear(true);
      cockpit2.localStorage.removeItem("login-data", true);
      cockpit2.localStorage.clear(false);
      if (reload !== false)
        transport_globals.reload_after_disconnect = true;
      ensure_transport(function(transport) {
        if (!transport.send_control({ command: "logout", disconnect: true }))
          window.location.reload(transport_globals.reload_after_disconnect);
      });
      window.sessionStorage.setItem("logout-intent", "explicit");
      if (reason)
        window.sessionStorage.setItem("logout-reason", reason);
    };
    cockpit2.drop_privileges = function drop_privileges() {
      console.warn("cockpit.drop_privileges() is deprecated");
      ensure_transport(function(transport) {
        transport.send_control({ command: "logout", disconnect: false });
      });
    };
    cockpit2.info = {};
    event_mixin(cockpit2.info, {});
    transport_globals.init_callback = function(options) {
      if (options.system) {
        cockpit2.info.ws = options.system;
        Object.assign(cockpit2.info, options.system);
      }
      if (options.system)
        cockpit2.info.dispatchEvent("changed");
      cockpit2.transport.options = options;
      cockpit2.transport.csrf_token = options["csrf-token"];
      cockpit2.transport.host = transport_globals.default_host;
    };
    let the_user = null;
    cockpit2.user = function() {
      if (!the_user) {
        const dbus = cockpit2.dbus(null, { bus: "internal" });
        return dbus.call(
          "/user",
          "org.freedesktop.DBus.Properties",
          "GetAll",
          ["cockpit.User"],
          { type: "s" }
        ).then(([user]) => {
          the_user = {
            id: user.Id.v,
            gid: user.Gid?.v,
            name: user.Name.v,
            full_name: user.Full.v,
            groups: user.Groups.v,
            home: user.Home.v,
            shell: user.Shell.v
          };
          Object.freeze(the_user);
          return the_user;
        }).finally(() => dbus.close());
      } else {
        return Promise.resolve(the_user);
      }
    };
    document.addEventListener("click", function(ev) {
      if (ev.target.classList && in_array(ev.target.classList, "disabled"))
        ev.stopPropagation();
    }, true);
    let last_loc = null;
    Object.defineProperty(cockpit2, "location", {
      enumerable: true,
      get: function() {
        if (!last_loc || last_loc.href !== window.location.hash.slice(1))
          last_loc = new Location();
        return last_loc;
      },
      set: function(v) {
        cockpit2.location.go(v);
      }
    });
    window.addEventListener("hashchange", function() {
      if (last_loc)
        last_loc.invalidate();
      last_loc = null;
      const hash = window.location.hash.slice(1);
      cockpit2.hint("location", { hash });
      cockpit2.dispatchEvent("locationchanged");
    });
    cockpit2.jump = function jump(path, host) {
      if (Array.isArray(path))
        path = "/" + path.map(encodeURIComponent).join("/").replaceAll("%40", "@").replaceAll("%3D", "=").replaceAll("%2B", "+");
      else
        path = "" + path;
      if (host === void 0)
        host = cockpit2.transport.host;
      const options = { command: "jump", location: path, host };
      cockpit2.transport.inject("\n" + JSON.stringify(options));
    };
    (function() {
      let hiddenHint = false;
      function visibility_change() {
        let value = document.hidden;
        if (value === false)
          value = hiddenHint;
        if (cockpit2.hidden !== value) {
          cockpit2.hidden = value;
          cockpit2.dispatchEvent("visibilitychange");
        }
      }
      document.addEventListener("visibilitychange", visibility_change);
      transport_globals.process_hints = function(data) {
        if ("hidden" in data) {
          hiddenHint = data.hidden;
          visibility_change();
        }
      };
      visibility_change();
    })();
    function ProcessError(options, name) {
      this.problem = options.problem || null;
      this.exit_status = options["exit-status"];
      if (this.exit_status === void 0)
        this.exit_status = null;
      this.exit_signal = options["exit-signal"];
      if (this.exit_signal === void 0)
        this.exit_signal = null;
      this.message = options.message;
      if (this.message === void 0) {
        if (this.problem)
          this.message = cockpit2.message(options.problem);
        else if (this.exit_signal !== null)
          this.message = cockpit2.format(_("$0 killed with signal $1"), name, this.exit_signal);
        else if (this.exit_status !== null)
          this.message = cockpit2.format(_("$0 exited with code $1"), name, this.exit_status);
        else
          this.message = cockpit2.format(_("$0 failed"), name);
      } else {
        this.message = this.message.trim();
      }
      this.toString = function() {
        return this.message;
      };
    }
    cockpit2.ProcessError = ProcessError;
    function spawn_debug() {
      if (window.debugging == "all" || window.debugging?.includes("spawn"))
        console.debug.apply(console, arguments);
    }
    cockpit2.spawn = function(command, options) {
      const dfd = cockpit2.defer();
      const args = { payload: "stream", spawn: [] };
      if (command instanceof Array) {
        for (let i = 0; i < command.length; i++)
          args.spawn.push(String(command[i]));
      } else {
        args.spawn.push(String(command));
      }
      if (options !== void 0)
        Object.assign(args, options);
      spawn_debug("process spawn:", JSON.stringify(args.spawn));
      const name = args.spawn[0] || "process";
      const channel = cockpit2.channel(args);
      const buffer = channel.buffer(null);
      channel.addEventListener("close", function(event, options2) {
        const data = buffer.squash();
        spawn_debug("process closed:", JSON.stringify(options2));
        if (data)
          spawn_debug("process output:", data);
        if (options2.message !== void 0)
          spawn_debug("process error:", options2.message);
        if (options2.problem)
          dfd.reject(new ProcessError(options2, name));
        else if (options2["exit-status"] || options2["exit-signal"])
          dfd.reject(new ProcessError(options2, name), data);
        else if (options2.message !== void 0)
          dfd.resolve(data, options2.message);
        else
          dfd.resolve(data);
      });
      const ret = dfd.promise;
      ret.stream = function(callback) {
        buffer.callback = callback.bind(ret);
        return this;
      };
      ret.input = function(message, stream) {
        if (message !== null && message !== void 0) {
          spawn_debug("process input:", message);
          iterate_data(message, function(data) {
            channel.send(data);
          });
        }
        if (!stream)
          channel.control({ command: "done" });
        return this;
      };
      ret.close = function(problem) {
        spawn_debug("process closing:", problem);
        if (channel.valid)
          channel.close(problem);
        return this;
      };
      return ret;
    };
    cockpit2.script = function(script, args, options) {
      if (!options && is_plain_object(args)) {
        options = args;
        args = [];
      }
      const command = ["/bin/sh", "-c", script, "--"];
      command.push.apply(command, args);
      return cockpit2.spawn(command, options);
    };
    function dbus_debug() {
      if (window.debugging == "all" || window.debugging?.includes("dbus"))
        console.debug.apply(console, arguments);
    }
    function DBusError(arg, arg1) {
      if (typeof arg == "string") {
        this.problem = arg;
        this.name = null;
        this.message = arg1 || cockpit2.message(arg);
      } else {
        this.problem = null;
        this.name = arg[0];
        this.message = arg[1][0] || arg[0];
      }
      this.toString = function() {
        return this.message;
      };
    }
    function DBusCache() {
      const self = this;
      let callbacks = [];
      self.data = {};
      self.meta = {};
      self.connect = function connect(path, iface, callback, first) {
        const cb = [path, iface, callback];
        if (first)
          callbacks.unshift(cb);
        else
          callbacks.push(cb);
        return {
          remove: function remove() {
            const length = callbacks.length;
            for (let i = 0; i < length; i++) {
              const cb2 = callbacks[i];
              if (cb2[0] === path && cb2[1] === iface && cb2[2] === callback) {
                delete cb2[i];
                break;
              }
            }
          }
        };
      };
      function emit(path, iface, props) {
        const copy = callbacks.slice();
        const length = copy.length;
        for (let i = 0; i < length; i++) {
          const cb = copy[i];
          if ((!cb[0] || cb[0] === path) && (!cb[1] || cb[1] === iface)) {
            cb[2](props, path);
          }
        }
      }
      self.update = function update(path, iface, props) {
        if (!self.data[path])
          self.data[path] = {};
        if (!self.data[path][iface])
          self.data[path][iface] = props;
        else
          props = Object.assign(self.data[path][iface], props);
        emit(path, iface, props);
      };
      self.remove = function remove(path, iface) {
        if (self.data[path]) {
          delete self.data[path][iface];
          emit(path, iface, null);
        }
      };
      self.lookup = function lookup(path, iface) {
        if (self.data[path])
          return self.data[path][iface];
        return void 0;
      };
      self.each = function each(iface, callback) {
        for (const path in self.data) {
          for (const ifa in self.data[path]) {
            if (ifa == iface)
              callback(self.data[path][iface], path);
          }
        }
      };
      self.close = function close() {
        self.data = {};
        const copy = callbacks;
        callbacks = [];
        const length = copy.length;
        for (let i = 0; i < length; i++)
          copy[i].callback();
      };
    }
    function DBusProxy(client, cache, iface, path, options) {
      const self = this;
      event_mixin(self, {});
      let valid = false;
      let defined = false;
      const waits = cockpit2.defer();
      Object.defineProperties(self, {
        client: { value: client, enumerable: false, writable: false },
        path: { value: path, enumerable: false, writable: false },
        iface: { value: iface, enumerable: false, writable: false },
        valid: { get: function() {
          return valid;
        }, enumerable: false },
        wait: {
          enumerable: false,
          writable: false,
          value: function(func) {
            if (func)
              waits.promise.always(func);
            return waits.promise;
          }
        },
        call: {
          value: function(name, args, options2) {
            return client.call(path, iface, name, args, options2);
          },
          enumerable: false,
          writable: false
        },
        data: { value: {}, enumerable: false }
      });
      if (!options)
        options = {};
      function define() {
        if (!cache.meta[iface])
          return;
        const meta = cache.meta[iface];
        defined = true;
        Object.keys(meta.methods || {}).forEach(function(name) {
          if (name[0].toLowerCase() == name[0])
            return;
          Object.defineProperty(self, name, {
            enumerable: false,
            value: function() {
              const dfd = cockpit2.defer();
              client.call(path, iface, name, Array.prototype.slice.call(arguments)).done(function(reply) {
                dfd.resolve.apply(dfd, reply);
              }).fail(function(ex) {
                dfd.reject(ex);
              });
              return dfd.promise;
            }
          });
        });
        Object.keys(meta.properties || {}).forEach(function(name) {
          if (name[0].toLowerCase() == name[0])
            return;
          const config = {
            enumerable: true,
            get: function() {
              return self.data[name];
            },
            set: function(v) {
              throw Error(name + "is not writable");
            }
          };
          const prop = meta.properties[name];
          if (prop.flags && prop.flags.indexOf("w") !== -1) {
            config.set = function(v) {
              client.call(
                path,
                "org.freedesktop.DBus.Properties",
                "Set",
                [iface, name, cockpit2.variant(prop.type, v)]
              ).fail(function(ex) {
                console.log("Couldn't set " + iface + " " + name + " at " + path + ": " + ex);
              });
            };
          }
          Object.defineProperty(self, name, config);
        });
      }
      function update(props) {
        if (props) {
          Object.assign(self.data, props);
          if (!defined)
            define();
          valid = true;
        } else {
          valid = false;
        }
        self.dispatchEvent("changed", props);
      }
      cache.connect(path, iface, update, true);
      update(cache.lookup(path, iface));
      function signal(path2, iface2, name, args) {
        self.dispatchEvent("signal", name, args);
        if (name[0].toLowerCase() != name[0]) {
          args = args.slice();
          args.unshift(name);
          self.dispatchEvent.apply(self, args);
        }
      }
      client.subscribe({ path, interface: iface }, signal, options.subscribe !== false);
      function waited(ex) {
        if (valid)
          waits.resolve();
        else
          waits.reject(ex);
      }
      if (options.watch !== false)
        client.watch({ path, interface: iface }).always(waited);
      else
        waited();
    }
    function DBusProxies(client, cache, iface, path_namespace, options) {
      const self = this;
      event_mixin(self, {});
      self.client = client;
      self.iface = iface;
      self.path_namespace = path_namespace;
      let waits;
      self.wait = function(func) {
        if (func)
          waits.always(func);
        return waits;
      };
      Object.defineProperties(self, {
        client: { enumerable: false, writable: false },
        iface: { enumerable: false, writable: false },
        path_namespace: { enumerable: false, writable: false },
        wait: { enumerable: false, writable: false }
      });
      const match = { interface: iface, path_namespace };
      client.subscribe(match);
      if (options.watch !== false) {
        waits = client.watch(match);
      } else {
        waits = cockpit2.defer().resolve().promise;
      }
      options = { watch: false, subscribe: false, ...options };
      function update(props, path) {
        let proxy = self[path];
        if (path) {
          if (!props && proxy) {
            delete self[path];
            self.dispatchEvent("removed", proxy);
          } else if (props) {
            if (!proxy) {
              proxy = self[path] = client.proxy(iface, path, options);
              self.dispatchEvent("added", proxy);
            }
            self.dispatchEvent("changed", proxy);
          }
        }
      }
      cache.connect(null, iface, update, false);
      cache.each(iface, update);
    }
    function DBusClient(name, options) {
      const self = this;
      event_mixin(self, {});
      const args = {};
      let track = false;
      let owner = null;
      if (options) {
        if (options.track)
          track = true;
        delete options.track;
        Object.assign(args, options);
      }
      args.payload = "dbus-json3";
      if (name)
        args.name = name;
      self.options = options;
      self.unique_name = null;
      dbus_debug("dbus open: ", args);
      let channel = cockpit2.channel(args);
      const subscribers = {};
      let calls = {};
      let cache;
      let closed;
      self.constructors = { "*": DBusProxy };
      self.wait = channel.wait;
      function ensure_cache() {
        if (!cache)
          cache = new DBusCache();
      }
      function send(payload) {
        if (channel?.valid) {
          dbus_debug("dbus:", payload);
          channel.send(payload);
          return true;
        }
        return false;
      }
      function matches(signal, match) {
        if (match.path && signal[0] !== match.path)
          return false;
        if (match.path_namespace && signal[0].indexOf(match.path_namespace) !== 0)
          return false;
        if (match.interface && signal[1] !== match.interface)
          return false;
        if (match.member && signal[2] !== match.member)
          return false;
        if (match.arg0 && (!signal[3] || signal[3][0] !== match.arg0))
          return false;
        return true;
      }
      function on_message(event, payload) {
        dbus_debug("dbus:", payload);
        let msg;
        try {
          msg = JSON.parse(payload);
        } catch (ex) {
          console.warn("received invalid dbus json message:", ex);
        }
        if (msg === void 0) {
          channel.close({ problem: "protocol-error" });
          return;
        }
        const dfd = msg.id !== void 0 ? calls[msg.id] : void 0;
        if (msg.reply) {
          if (dfd) {
            const options2 = {};
            if (msg.type)
              options2.type = msg.type;
            if (msg.flags)
              options2.flags = msg.flags;
            dfd.resolve(msg.reply[0] || [], options2);
            delete calls[msg.id];
          }
          return;
        } else if (msg.error) {
          if (dfd) {
            dfd.reject(new DBusError(msg.error));
            delete calls[msg.id];
          }
          return;
        }
        later_invoke(function() {
          if (msg.signal) {
            for (const id in subscribers) {
              const subscription = subscribers[id];
              if (subscription.callback) {
                if (matches(msg.signal, subscription.match))
                  subscription.callback.apply(self, msg.signal);
              }
            }
          } else if (msg.notify) {
            notify(msg.notify);
          } else if (msg.meta) {
            meta(msg.meta);
          } else if (msg.owner !== void 0) {
            self.dispatchEvent("owner", msg.owner);
            if (track && owner)
              self.close();
            owner = msg.owner;
          } else {
            dbus_debug("received unexpected dbus json message:", payload);
          }
        });
      }
      function meta(data) {
        ensure_cache();
        Object.assign(cache.meta, data);
        self.dispatchEvent("meta", data);
      }
      function notify(data) {
        ensure_cache();
        for (const path in data) {
          for (const iface in data[path]) {
            const props = data[path][iface];
            if (!props)
              cache.remove(path, iface);
            else
              cache.update(path, iface, props);
          }
        }
        self.dispatchEvent("notify", data);
      }
      this.notify = notify;
      function close_perform(options2) {
        closed = options2.problem || "disconnected";
        const outstanding = calls;
        calls = {};
        for (const id in outstanding) {
          outstanding[id].reject(new DBusError(closed, options2.message));
        }
        self.dispatchEvent("close", options2);
      }
      this.close = function close(options2) {
        if (typeof options2 == "string")
          options2 = { problem: options2 };
        if (!options2)
          options2 = {};
        if (channel)
          channel.close(options2);
        else
          close_perform(options2);
      };
      function on_ready(event, message) {
        dbus_debug("dbus ready:", options);
        self.unique_name = message["unique-name"];
      }
      function on_close(event, options2) {
        dbus_debug("dbus close:", options2);
        channel.removeEventListener("ready", on_ready);
        channel.removeEventListener("message", on_message);
        channel.removeEventListener("close", on_close);
        channel = null;
        close_perform(options2);
      }
      channel.addEventListener("ready", on_ready);
      channel.addEventListener("message", on_message);
      channel.addEventListener("close", on_close);
      let last_cookie = 1;
      this.call = function call(path, iface, method, args2, options2) {
        const dfd = cockpit2.defer();
        const id = String(last_cookie);
        last_cookie++;
        const method_call = {
          ...options2,
          call: [path, iface, method, args2 || []],
          id
        };
        const msg = JSON.stringify(method_call);
        if (send(msg))
          calls[id] = dfd;
        else
          dfd.reject(new DBusError(closed));
        return dfd.promise;
      };
      self.signal = function signal(path, iface, member, args2, options2) {
        if (!channel || !channel.valid)
          return;
        const message = { ...options2, signal: [path, iface, member, args2 || []] };
        send(JSON.stringify(message));
      };
      this.subscribe = function subscribe(match, callback, rule) {
        const subscription = {
          match: { ...match },
          callback
        };
        if (rule !== false)
          send(JSON.stringify({ "add-match": subscription.match }));
        let id;
        if (callback) {
          id = String(last_cookie);
          last_cookie++;
          subscribers[id] = subscription;
        }
        return {
          remove: function() {
            let prev;
            if (id) {
              prev = subscribers[id];
              if (prev)
                delete subscribers[id];
            }
            if (rule !== false && prev)
              send(JSON.stringify({ "remove-match": prev.match }));
          }
        };
      };
      self.watch = function watch(path) {
        const match = is_plain_object(path) ? { ...path } : { path: String(path) };
        const id = String(last_cookie);
        last_cookie++;
        const dfd = cockpit2.defer();
        const msg = JSON.stringify({ watch: match, id });
        if (send(msg))
          calls[id] = dfd;
        else
          dfd.reject(new DBusError(closed));
        const ret = dfd.promise;
        ret.remove = function remove() {
          if (id in calls) {
            dfd.reject(new DBusError("cancelled"));
            delete calls[id];
          }
          send(JSON.stringify({ unwatch: match }));
        };
        return ret;
      };
      self.proxy = function proxy(iface, path, options2) {
        if (!iface)
          iface = name;
        iface = String(iface);
        if (!path)
          path = "/" + iface.replaceAll(".", "/");
        let Constructor = self.constructors[iface];
        if (!Constructor)
          Constructor = self.constructors["*"];
        if (!options2)
          options2 = {};
        ensure_cache();
        return new Constructor(self, cache, iface, String(path), options2);
      };
      self.proxies = function proxies(iface, path_namespace, options2) {
        if (!iface)
          iface = name;
        if (!path_namespace)
          path_namespace = "/";
        if (!options2)
          options2 = {};
        ensure_cache();
        return new DBusProxies(self, cache, String(iface), String(path_namespace), options2);
      };
    }
    const shared_dbus = {
      internal: null,
      session: null,
      system: null
    };
    cockpit2.dbus = function dbus(name, options) {
      if (!options)
        options = { bus: "system" };
      const keys = Object.keys(options);
      const bus = options.bus;
      const shared = !name && keys.length == 1 && bus in shared_dbus;
      if (shared && shared_dbus[bus])
        return shared_dbus[bus];
      const client = new DBusClient(name, options);
      if (shared) {
        const old_close = client.close;
        client.close = function() {
          if (arguments.length > 0)
            old_close.apply(client, arguments);
        };
        client.addEventListener("close", function() {
          if (shared_dbus[bus] == client)
            shared_dbus[bus] = null;
        });
        shared_dbus[bus] = client;
      }
      return client;
    };
    cockpit2.variant = function variant(type, value) {
      return { v: value, t: type };
    };
    cockpit2.byte_array = function byte_array(string) {
      console.warn("cockpit.byte_array() is deprecated, use window.btoa");
      return window.btoa(string);
    };
    cockpit2.file = function file(path, options) {
      options = options || {};
      const binary = options.binary;
      const self = {
        path,
        read,
        replace,
        modify,
        watch,
        close
      };
      const base_channel_options = { ...options };
      delete base_channel_options.syntax;
      function parse(str) {
        if (options.syntax?.parse)
          return options.syntax.parse(str);
        else
          return str;
      }
      function stringify(obj) {
        if (options.syntax?.stringify)
          return options.syntax.stringify(obj);
        else
          return obj;
      }
      let read_promise = null;
      let read_channel;
      function read() {
        if (read_promise)
          return read_promise;
        const dfd = cockpit2.defer();
        const opts = {
          ...base_channel_options,
          payload: "fsread1",
          path
        };
        function try_read() {
          read_channel = cockpit2.channel(opts);
          const content_parts = [];
          read_channel.addEventListener("message", function(event, message) {
            content_parts.push(message);
          });
          read_channel.addEventListener("close", function(event, message) {
            read_channel = null;
            if (message.problem == "change-conflict") {
              try_read();
              return;
            }
            read_promise = null;
            if (message.problem) {
              const error = new BasicError(message.problem, message.message);
              fire_watch_callbacks(null, null, error);
              dfd.reject(error);
              return;
            }
            let content;
            if (message.tag == "-")
              content = null;
            else {
              try {
                content = parse(join_data(content_parts, binary));
              } catch (e) {
                fire_watch_callbacks(null, null, e);
                dfd.reject(e);
                return;
              }
            }
            fire_watch_callbacks(content, message.tag);
            dfd.resolve(content, message.tag);
          });
        }
        try_read();
        read_promise = dfd.promise;
        return read_promise;
      }
      let replace_channel = null;
      function replace(new_content, expected_tag) {
        const dfd = cockpit2.defer();
        let file_content;
        try {
          file_content = new_content === null ? null : stringify(new_content);
        } catch (e) {
          dfd.reject(e);
          return dfd.promise;
        }
        if (replace_channel)
          replace_channel.close("abort");
        const opts = {
          ...base_channel_options,
          payload: "fsreplace1",
          path,
          tag: expected_tag
        };
        replace_channel = cockpit2.channel(opts);
        replace_channel.addEventListener("close", function(event, message) {
          replace_channel = null;
          if (message.problem) {
            dfd.reject(new BasicError(message.problem, message.message));
          } else {
            fire_watch_callbacks(new_content, message.tag);
            dfd.resolve(message.tag);
          }
        });
        if (file_content !== null) {
          if (file_content.length === 0 || file_content.byteLength === 0) {
            replace_channel.send(file_content);
          } else {
            iterate_data(file_content, (data) => {
              replace_channel.send(data);
            });
          }
        }
        replace_channel.control({ command: "done" });
        return dfd.promise;
      }
      function modify(callback, initial_content, initial_tag) {
        const dfd = cockpit2.defer();
        function update(content, tag) {
          let new_content = callback(content);
          if (new_content === void 0)
            new_content = content;
          replace(new_content, tag).done(function(new_tag) {
            dfd.resolve(new_content, new_tag);
          }).fail(function(error) {
            if (error.problem == "change-conflict")
              read_then_update();
            else
              dfd.reject(error);
          });
        }
        function read_then_update() {
          read().done(update).fail(function(error) {
            dfd.reject(error);
          });
        }
        if (initial_content === void 0)
          read_then_update();
        else
          update(initial_content, initial_tag);
        return dfd.promise;
      }
      const watch_callbacks = [];
      let n_watch_callbacks = 0;
      let watch_channel = null;
      let watch_tag;
      function ensure_watch_channel(options2) {
        if (n_watch_callbacks > 0) {
          if (watch_channel)
            return;
          watch_channel = new FsInfoClient(path, ["tag"], { superuser: base_channel_options.superuser });
          watch_channel.on("change", (state) => {
            if (state.error) {
              if (state.error.problem === "not-found") {
                fire_watch_callbacks(null, "-");
              } else {
                const error = new BasicError(state.error.problem, state.error.message);
                fire_watch_callbacks(null, null, error);
              }
            } else if (state.info && state.info.tag) {
              if (state.info.tag !== watch_tag) {
                if (options2?.read === false)
                  fire_watch_callbacks(null, state.info.tag);
                else
                  read();
              }
            }
          });
          watch_channel.on("close", (ex) => {
            if (ex.problem === "not-supported") {
              const opts = {
                payload: "fswatch1",
                path,
                superuser: base_channel_options.superuser
              };
              watch_channel = cockpit2.channel(opts);
              watch_channel.addEventListener("message", (event, message_string) => {
                let message;
                try {
                  message = JSON.parse(message_string);
                } catch (e) {
                  message = null;
                }
                if (message && message.path == path && message.tag && message.tag != watch_tag) {
                  if (options2 && options2.read !== void 0 && !options2.read)
                    fire_watch_callbacks(null, message.tag);
                  else
                    read();
                }
              });
              read();
            }
          });
        } else {
          if (watch_channel) {
            watch_channel.close();
            watch_channel = null;
          }
        }
      }
      function fire_watch_callbacks() {
        watch_tag = arguments[1] || null;
        invoke_functions(watch_callbacks, self, arguments);
      }
      function watch(callback, options2) {
        if (callback)
          watch_callbacks.push(callback);
        n_watch_callbacks += 1;
        ensure_watch_channel(options2);
        watch_tag = null;
        return {
          remove: function() {
            if (callback) {
              const index = watch_callbacks.indexOf(callback);
              if (index > -1)
                watch_callbacks[index] = null;
            }
            n_watch_callbacks -= 1;
            ensure_watch_channel(options2);
          }
        };
      }
      function close() {
        if (read_channel)
          read_channel.close("cancelled");
        if (replace_channel)
          replace_channel.close("cancelled");
        if (watch_channel)
          watch_channel.close();
      }
      return self;
    };
    let po_data = {};
    let po_plural;
    cockpit2.language = "en";
    cockpit2.language_direction = "ltr";
    const test_l10n = window.localStorage.test_l10n;
    cockpit2.locale = function locale(po) {
      let lang = cockpit2.language;
      let lang_dir = cockpit2.language_direction;
      let header;
      if (po) {
        Object.assign(po_data, po);
        header = po[""];
      } else if (po === null) {
        po_data = {};
      }
      if (header) {
        if (header["plural-forms"])
          po_plural = header["plural-forms"];
        if (header.language)
          lang = header.language;
        if (header["language-direction"])
          lang_dir = header["language-direction"];
      }
      cockpit2.language = lang;
      cockpit2.language_direction = lang_dir;
    };
    cockpit2.translate = function translate() {
      let what;
      if (arguments.length === 0)
        what = [document];
      else if (arguments.length === 1 && arguments[0].length)
        what = arguments[0];
      else
        what = arguments;
      const wlen = what.length;
      for (let w = 0; w < wlen; w++) {
        let list = null;
        if (what[w].querySelectorAll)
          list = what[w].querySelectorAll("[translate]");
        if (!list)
          continue;
        for (let i = 0; i < list.length; i++) {
          const el = list[i];
          let val = el.getAttribute("translate") || "yes";
          if (val == "no")
            continue;
          const tasks = val.split(" ");
          val = el.getAttribute("translate-context") || el.getAttribute("context");
          for (let t = 0; t < tasks.length; t++) {
            if (tasks[t] == "yes" || tasks[t] == "translate")
              el.textContent = cockpit2.gettext(val, el.textContent);
            else if (tasks[t])
              el.setAttribute(tasks[t], cockpit2.gettext(val, el.getAttribute(tasks[t]) || ""));
          }
          el.removeAttribute("translate");
        }
      }
    };
    cockpit2.gettext = function gettext(context, string) {
      if (arguments.length == 1) {
        string = context;
        context = void 0;
      }
      const key = context ? context + "" + string : string;
      if (po_data) {
        const translated = po_data[key];
        if (translated?.[1])
          string = translated[1];
      }
      if (test_l10n === "true")
        return "\xBB" + string + "\xAB";
      return string;
    };
    function imply(val) {
      return val === true ? 1 : val || 0;
    }
    cockpit2.ngettext = function ngettext(context, string1, stringN, num) {
      if (arguments.length == 3) {
        num = stringN;
        stringN = string1;
        string1 = context;
        context = void 0;
      }
      const key = context ? context + "" + string1 : string1;
      if (po_data && po_plural) {
        const translated = po_data[key];
        if (translated) {
          const i = imply(po_plural(num)) + 1;
          if (translated[i])
            return translated[i];
        }
      }
      if (num == 1)
        return string1;
      return stringN;
    };
    cockpit2.noop = function noop(arg0, arg1) {
      return arguments[arguments.length - 1];
    };
    const _ = cockpit2.gettext;
    cockpit2.message = function message(arg) {
      if (arg.message)
        return arg.message;
      let problem = null;
      if (arg.problem)
        problem = arg.problem;
      else
        problem = arg + "";
      if (problem == "terminated")
        return _("Your session has been terminated.");
      else if (problem == "no-session")
        return _("Your session has expired. Please log in again.");
      else if (problem == "access-denied")
        return _("Not permitted to perform this action.");
      else if (problem == "authentication-failed")
        return _("Login failed");
      else if (problem == "authentication-not-supported")
        return _("The server refused to authenticate using any supported methods.");
      else if (problem == "unknown-hostkey")
        return _("Untrusted host");
      else if (problem == "unknown-host")
        return _("Untrusted host");
      else if (problem == "invalid-hostkey")
        return _("Host key is incorrect");
      else if (problem == "internal-error")
        return _("Internal error");
      else if (problem == "timeout")
        return _("Connection has timed out.");
      else if (problem == "no-cockpit")
        return _("Cockpit is not installed on the system.");
      else if (problem == "no-forwarding")
        return _("Cannot forward login credentials");
      else if (problem == "disconnected")
        return _("Server has closed the connection.");
      else if (problem == "not-supported")
        return _("Cockpit is not compatible with the software on the system.");
      else if (problem == "no-host")
        return _("Cockpit could not contact the given host.");
      else if (problem == "too-large")
        return _("Too much data");
      else
        return problem;
    };
    function HttpError(arg0, arg1, message) {
      this.status = parseInt(arg0, 10);
      this.reason = arg1;
      this.message = message || arg1;
      this.problem = null;
      this.valueOf = function() {
        return this.status;
      };
      this.toString = function() {
        return this.status + " " + this.message;
      };
    }
    function http_debug() {
      if (window.debugging == "all" || window.debugging?.includes("http"))
        console.debug.apply(console, arguments);
    }
    function find_header(headers, name) {
      if (!headers)
        return void 0;
      name = name.toLowerCase();
      for (const head in headers) {
        if (head.toLowerCase() == name)
          return headers[head];
      }
      return void 0;
    }
    function HttpClient(endpoint, options) {
      const self = this;
      self.options = options;
      options.payload = "http-stream2";
      const active_requests = [];
      if (endpoint !== void 0) {
        if (endpoint.indexOf && endpoint.indexOf("/") === 0) {
          options.unix = endpoint;
        } else {
          const port = parseInt(endpoint, 10);
          if (!isNaN(port))
            options.port = port;
          else
            throw Error("The endpoint must be either a unix path or port number");
        }
      }
      if (options.address) {
        if (!options.capabilities)
          options.capabilities = [];
        options.capabilities.push("address");
      }
      function param(obj) {
        return Object.keys(obj).map(function(k) {
          return encodeURIComponent(k) + "=" + encodeURIComponent(obj[k]);
        }).join("&").split("%20").join("+");
      }
      self.request = function request(req) {
        const dfd = cockpit2.defer();
        const ret = dfd.promise;
        if (!req.path)
          req.path = "/";
        if (!req.method)
          req.method = "GET";
        if (req.params) {
          if (req.path.indexOf("?") === -1)
            req.path += "?" + param(req.params);
          else
            req.path += "&" + param(req.params);
        }
        delete req.params;
        const input = req.body;
        delete req.body;
        const headers = req.headers;
        delete req.headers;
        Object.assign(req, options);
        if (options.headers && headers)
          req.headers = { ...options.headers, ...headers };
        else if (options.headers)
          req.headers = options.headers;
        else
          req.headers = headers;
        http_debug("http request:", JSON.stringify(req));
        const channel = cockpit2.channel(req);
        if (input !== void 0) {
          if (input !== "") {
            http_debug("http input:", input);
            iterate_data(input, function(data) {
              channel.send(data);
            });
          }
          http_debug("http", req.method, req.path, "request sent, channel done");
          channel.control({ command: "done" });
        }
        let streamer = null;
        let responsers = null;
        let resp = null;
        const buffer = channel.buffer(function(data) {
          if (resp && resp.status >= 200 && resp.status <= 299 && streamer)
            return streamer.call(ret, data);
          return 0;
        });
        function on_control(event, options2) {
          if (options2.command == "response") {
            resp = options2;
            if (responsers) {
              resp.headers = resp.headers || {};
              invoke_functions(responsers, ret, [resp.status, resp.headers]);
            }
          }
        }
        function on_close(event, options2) {
          const pos = active_requests.indexOf(ret);
          if (pos >= 0)
            active_requests.splice(pos, 1);
          if (options2.problem) {
            http_debug("http problem: ", options2.problem);
            dfd.reject(new BasicError(options2.problem, options2.message));
          } else {
            const body = buffer.squash();
            if (resp && (resp.status < 200 || resp.status > 299)) {
              let message;
              const type = find_header(resp.headers, "Content-Type");
              if (type && !channel.binary) {
                if (type.indexOf("text/plain") === 0)
                  message = body;
              }
              http_debug("http", req.method, req.path, "failed:", resp.status, resp.reason);
              dfd.reject(new HttpError(resp.status, resp.reason, message), body);
            } else {
              if (resp)
                http_debug("http", req.method, req.path, "succeeded:", resp.status);
              else
                http_debug("http", req.method, req.path, "failed without response");
              dfd.resolve(body);
            }
          }
          channel.removeEventListener("control", on_control);
          channel.removeEventListener("close", on_close);
        }
        channel.addEventListener("control", on_control);
        channel.addEventListener("close", on_close);
        ret.stream = function(callback) {
          streamer = callback;
          return ret;
        };
        ret.response = function(callback) {
          if (responsers === null)
            responsers = [];
          responsers.push(callback);
          return ret;
        };
        ret.input = function(message, stream) {
          if (message !== null && message !== void 0) {
            http_debug("http input:", message);
            iterate_data(message, function(data) {
              channel.send(data);
            });
          }
          if (!stream) {
            http_debug("http done");
            channel.control({ command: "done" });
          }
          return ret;
        };
        ret.close = function(problem) {
          http_debug("http closing:", problem);
          channel.close(problem);
          return ret;
        };
        active_requests.push(ret);
        return ret;
      };
      self.get = function get(path, params, headers) {
        return self.request({
          method: "GET",
          params,
          path,
          body: "",
          headers
        });
      };
      self.post = function post(path, body, headers) {
        headers = headers || {};
        if (is_plain_object(body) || Array.isArray(body)) {
          body = JSON.stringify(body);
          if (find_header(headers, "Content-Type") === void 0)
            headers["Content-Type"] = "application/json";
        } else if (body === void 0 || body === null) {
          body = "";
        } else if (typeof body !== "string") {
          body = String(body);
        }
        return self.request({
          method: "POST",
          path,
          body,
          headers
        });
      };
      self.close = function close(problem) {
        const reqs = active_requests.slice();
        for (let i = 0; i < reqs.length; i++)
          reqs[i].close(problem);
      };
    }
    cockpit2.http = function(endpoint, options) {
      if (is_plain_object(endpoint) && options === void 0) {
        options = endpoint;
        endpoint = void 0;
      }
      return new HttpClient(endpoint, options || {});
    };
    function check_superuser() {
      return new Promise((resolve, reject) => {
        const ch = cockpit2.channel({ payload: "null", superuser: "require" });
        ch.wait().then(() => resolve(true)).catch(() => resolve(false)).always(() => ch.close());
      });
    }
    function Permission(options) {
      const self = this;
      event_mixin(self, {});
      const api = cockpit2.dbus(null, { bus: "internal" }).proxy("cockpit.Superuser", "/superuser");
      api.addEventListener("changed", maybe_reload);
      function maybe_reload() {
        if (api.valid && self.allowed !== null) {
          if (self.allowed != (api.Current != "none"))
            window.location.reload(true);
        }
      }
      self.allowed = null;
      self.user = options ? options.user : null;
      self.is_superuser = options ? options._is_superuser : null;
      let group = null;
      let admin = false;
      if (options)
        group = options.group;
      if (options?.admin)
        admin = true;
      function decide(user) {
        if (user.id === 0)
          return true;
        if (group)
          return !!(user.groups || []).includes(group);
        if (admin)
          return self.is_superuser;
        if (user.id === void 0)
          return null;
        return false;
      }
      if (self.user && self.is_superuser !== null) {
        self.allowed = decide(self.user);
      } else {
        Promise.all([cockpit2.user(), check_superuser()]).then(([user, is_superuser]) => {
          self.user = user;
          self.is_superuser = is_superuser;
          const allowed = decide(user);
          if (self.allowed !== allowed) {
            self.allowed = allowed;
            maybe_reload();
            self.dispatchEvent("changed");
          }
        });
      }
      self.close = function close() {
      };
    }
    cockpit2.permission = function permission(arg) {
      return new Permission(arg);
    };
    function MetricsChannel(interval, options_list, cache) {
      const self = this;
      event_mixin(self, {});
      if (options_list.length === void 0)
        options_list = [options_list];
      const channels = [];
      let following = false;
      self.series = cockpit2.series(interval, cache, fetch_for_series);
      self.archives = null;
      self.meta = null;
      function fetch_for_series(beg, end, for_walking) {
        if (!for_walking)
          self.fetch(beg, end);
        else
          self.follow();
      }
      function transfer(options_list2, callback, is_archive) {
        if (options_list2.length === 0)
          return;
        if (!is_archive) {
          if (following)
            return;
          following = true;
        }
        const options = {
          payload: "metrics1",
          interval,
          source: "internal",
          ...options_list2[0]
        };
        delete options.archive_source;
        const channel = cockpit2.channel(options);
        channels.push(channel);
        let meta = null;
        let last = null;
        let beg;
        channel.addEventListener("close", function(ev, close_options) {
          if (!is_archive)
            following = false;
          if (options_list2.length > 1 && (close_options.problem == "not-supported" || close_options.problem == "not-found")) {
            transfer(options_list2.slice(1), callback);
          } else if (close_options.problem) {
            if (close_options.problem != "terminated" && close_options.problem != "disconnected" && close_options.problem != "authentication-failed" && (close_options.problem != "not-found" || !is_archive) && (close_options.problem != "not-supported" || !is_archive)) {
              console.warn("metrics channel failed: " + close_options.problem);
            }
          } else if (is_archive) {
            if (!self.archives) {
              self.archives = true;
              self.dispatchEvent("changed");
            }
          }
        });
        channel.addEventListener("message", function(ev, payload) {
          const message = JSON.parse(payload);
          const message_len = message.length;
          if (message_len === void 0) {
            meta = message;
            let timestamp = 0;
            if (meta.now && meta.timestamp)
              timestamp = meta.timestamp + (Date.now() - meta.now);
            beg = Math.floor(timestamp / interval);
            callback(beg, meta, null, options_list2[0]);
            self.meta = meta;
            self.dispatchEvent("changed");
          } else if (meta) {
            for (let i = 0; i < message_len; i++) {
              const data = message[i];
              if (last) {
                for (let j = 0; j < last.length; j++) {
                  const dataj = data[j];
                  if (dataj === null || dataj === void 0) {
                    data[j] = last[j];
                  } else {
                    const dataj_len = dataj.length;
                    if (dataj_len !== void 0) {
                      const lastj = last[j];
                      const lastj_len = last[j].length;
                      let k;
                      for (k = 0; k < dataj_len; k++) {
                        if (dataj[k] === null)
                          dataj[k] = lastj[k];
                      }
                      for (; k < lastj_len; k++)
                        dataj[k] = lastj[k];
                    }
                  }
                }
              }
              last = data;
            }
            callback(beg, meta, message, options_list2[0]);
            beg += message_len;
            meta.timestamp += interval * message_len;
          }
        });
      }
      function drain(beg, meta, message, options) {
        let mapping = meta.mapping;
        if (!mapping) {
          mapping = {};
          meta.metrics.forEach(function(metric, i) {
            const map = { "": i };
            const name = options.metrics_path_names?.[i] ?? metric.name;
            mapping[name] = map;
            if (metric.instances) {
              metric.instances.forEach(function(instance, i2) {
                if (instance === "")
                  instance = "/";
                map[instance] = { "": i2 };
              });
            }
          });
          meta.mapping = mapping;
        }
        if (message)
          self.series.input(beg, message, mapping);
      }
      self.fetch = function fetch(beg, end) {
        const timestamp = beg * interval - Date.now();
        const limit = end - beg;
        const archive_options_list = [];
        for (let i = 0; i < options_list.length; i++) {
          if (options_list[i].archive_source) {
            archive_options_list.push({
              ...options_list[i],
              source: options_list[i].archive_source,
              timestamp,
              limit
            });
          }
        }
        transfer(archive_options_list, drain, true);
      };
      self.follow = function follow() {
        transfer(options_list, drain);
      };
      self.close = function close(options) {
        const len = channels.length;
        if (self.series)
          self.series.close();
        for (let i = 0; i < len; i++)
          channels[i].close(options);
      };
    }
    cockpit2.metrics = function metrics(interval, options) {
      return new MetricsChannel(interval, options);
    };
    cockpit2.oops = function oops() {
      if (window.parent !== window && window.name.indexOf("cockpit1:") === 0)
        window.parent.postMessage('\n{ "command": "oops" }', transport_origin);
    };
    const old_onerror = window.onerror;
    window.onerror = function(msg, url, line) {
      console.log("cockpit onerror", msg, url, line);
      console.log(console.trace());
      if (url != "")
        cockpit2.oops();
      if (old_onerror)
        return old_onerror(msg, url, line);
      return false;
    };
    cockpit2.assert = (predicate, message) => {
      if (!predicate) {
        throw new Error(`Assertion failed: ${message}`);
      }
    };
    return cockpit2;
  }
  var cockpit = factory();
  window.cockpit = cockpit;
})();
//# sourceMappingURL=cockpit.js.map
